package de.duehl.basics.text.data;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Diese Klasse stellt einen Suchtext und die Stelle, an der er gefunden wurde, dar.
 *
 * @version 1.01     2018-10-25
 * @author Christian Dühl
 */

public class FoundSearch implements Comparable<FoundSearch> {

    private final int index;

    private final String search;

    public FoundSearch(int index, String search) {
        this.index = index;
        this.search = search;
    }

    public int getIndex() {
        return index;
    }

    public String getSearch() {
        return search;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + index;
        result = prime * result + ((search == null) ? 0 : search.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        FoundSearch other = (FoundSearch) obj;
        if (index != other.index) {
            return false;
        }
        if (search == null) {
            if (other.search != null) {
                return false;
            }
        }
        else if (!search.equals(other.search)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "FoundSearch [index=" + index + ", search=" + search + "]";
    }

    public static FoundSearch notFound() {
        return new FoundSearch(-1, "SUCHBEGRIFF LIEGT NICHT VOR!");
    }

    @Override
    public int compareTo(FoundSearch that) {
        return this.index - that.index;
    }

}
