package de.duehl.basics.debug;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import org.junit.Ignore;
import org.junit.Test;

import de.duehl.basics.system.OutputStreamSaveAndRelease;
import de.duehl.basics.test.OutAndErrStreamHandlingTest;
import de.duehl.basics.text.Text;

public class DebugHelperTest extends OutAndErrStreamHandlingTest {

    @Test
    public void say() {
        DebugHelper.say();
        String actual = getOutContents();
        String expected = Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void sayWithText() {
        DebugHelper.say("Hello World!");
        String actual = getOutContents();
        String expected = "Hello World!" + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void sayWithoutLineBreak() {
        DebugHelper.sayWithoutLineBreak("Hello World!");
        String actual = getOutContents();
        String expected = "Hello World!";
        assertEquals(expected, actual);
    }

    @Test
    public void line() {
        DebugHelper.line();
        String actual = getOutContents();
        String expected = Text.multipleString("-", 80) + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void line10() {
        DebugHelper.line(10);
        String actual = getOutContents();
        String expected = Text.multipleString("-", 10) + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void embedTextIntoLine() {
        DebugHelper.embedTextIntoLine("foo");
        String actual = getOutContents();
        String expected = Text.multipleString("-", 37) + " foo " + Text.multipleString("-", 38)
                + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void embedTextIntoLine20() {
        DebugHelper.embedTextIntoLine("foo", 20);
        String actual = getOutContents();
        String expected = Text.multipleString("-", 7) + " foo " + Text.multipleString("-", 8)
                + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    /*
     * Das klappt nicht über das JUnit-Testsystem, da ist der dritte Index dann leider
     * NativeMethodAccessorImpl#invoke0() ist und nicht die wirklich aufrufende Methode
     * sayWithClassAndMethod().
     */
    @Ignore
    @Test
    public void sayWithClassAndMethod() {
        String className = "DebugHelperTest";
        String methodName = "sayWithClassAndMethod";
        String message = "foo bar baz.";

        OutputStreamSaveAndRelease ossr = new OutputStreamSaveAndRelease();
        ossr.saveSystemOut();
        DebugHelper.sayWithClassAndMethod(message);
        String actual = ossr.getOutContents();
        ossr.releaseSystemOut();

        String expected = className + "#" + methodName + "(): " + methodName + "\n";

        assertEquals(expected, actual);
    }

    @Test
    public void stackTraceToString() {
        String actual = DebugHelper.stackTraceToString();
        String expected = "java.base/java.lang.Thread.getStackTrace(Thread.java:1602)\n"
                + "de.duehl.basics.debug.DebugHelper.stackTraceToString(DebugHelper.java:35)\n"
                + "de.duehl.basics.debug.DebugHelperTest.stackTraceToString(DebugHelperTest.java:108)\n"
                + "java.base/jdk.internal.reflect.NativeMethodAccessorImpl.invoke0(Native Method)\n"
                + "java.base/jdk.internal.reflect.NativeMethodAccessorImpl.invoke(NativeMethodAccessorImpl.java:62)\n"
                + "java.base/jdk.internal.reflect.DelegatingMethodAccessorImpl.invoke(DelegatingMethodAccessorImpl.java:43)\n"
                + "java.base/java.lang.reflect.Method.invoke(Method.java:567)\n"
                + "org.junit.runners.model.FrameworkMethod$1.runReflectiveCall(FrameworkMethod.java:50)\n"
                + "org.junit.internal.runners.model.ReflectiveCallable.run(ReflectiveCallable.java:12)\n"
                + "org.junit.runners.model.FrameworkMethod.invokeExplosively(FrameworkMethod.java:47)\n"
                + "org.junit.internal.runners.statements.InvokeMethod.evaluate(InvokeMethod.java:17)\n"
                + "org.junit.internal.runners.statements.RunBefores.evaluate(RunBefores.java:26)\n"
                + "org.junit.internal.runners.statements.RunAfters.evaluate(RunAfters.java:27)\n"
                + "org.junit.runners.ParentRunner.runLeaf(ParentRunner.java:325)\n"
                + "org.junit.runners.BlockJUnit4ClassRunner.runChild(BlockJUnit4ClassRunner.java:78)\n"
                + "org.junit.runners.BlockJUnit4ClassRunner.runChild(BlockJUnit4ClassRunner.java:57)\n"
                + "org.junit.runners.ParentRunner$3.run(ParentRunner.java:290)\n"
                + "org.junit.runners.ParentRunner$1.schedule(ParentRunner.java:71)\n"
                + "org.junit.runners.ParentRunner.runChildren(ParentRunner.java:288)\n"
                + "org.junit.runners.ParentRunner.access$000(ParentRunner.java:58)\n"
                + "org.junit.runners.ParentRunner$2.evaluate(ParentRunner.java:268)\n"
                + "org.junit.runners.ParentRunner.run(ParentRunner.java:363)\n"
                + "org.eclipse.jdt.internal.junit4.runner.JUnit4TestReference.run(JUnit4TestReference.java:89)\n"
                + "org.eclipse.jdt.internal.junit.runner.TestExecution.run(TestExecution.java:41)\n"
                + "org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.runTests(RemoteTestRunner.java:541)\n"
                + "org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.runTests(RemoteTestRunner.java:763)\n"
                + "org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.run(RemoteTestRunner.java:463)\n"
                + "org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.main(RemoteTestRunner.java:209)\n";
        assertEquals(expected, actual);
    }

    @Test
    public void stackTraceToStringSmall() {
        String actual = DebugHelper.stackTraceToStringSmall();
        String expected = "DebugHelperTest stackTraceToStringSmall() 142 - (DebugHelperTest.java:142)\n"
                + "NativeMethodAccessorImpl invoke0() - (NativeMethodAccessorImpl.java)\n"
                + "NativeMethodAccessorImpl invoke() 62 - (NativeMethodAccessorImpl.java:62)\n"
                + "DelegatingMethodAccessorImpl invoke() 43 - (DelegatingMethodAccessorImpl.java:43)\n"
                + "Method invoke() 567 - (Method.java:567)\n"
                + "FrameworkMethod$1 runReflectiveCall() 50 - (FrameworkMethod.java:50)\n"
                + "ReflectiveCallable run() 12 - (ReflectiveCallable.java:12)\n"
                + "FrameworkMethod invokeExplosively() 47 - (FrameworkMethod.java:47)\n"
                + "InvokeMethod evaluate() 17 - (InvokeMethod.java:17)\n"
                + "RunBefores evaluate() 26 - (RunBefores.java:26)\n"
                + "RunAfters evaluate() 27 - (RunAfters.java:27)\n"
                + "ParentRunner runLeaf() 325 - (ParentRunner.java:325)\n"
                + "BlockJUnit4ClassRunner runChild() 78 - (BlockJUnit4ClassRunner.java:78)\n"
                + "BlockJUnit4ClassRunner runChild() 57 - (BlockJUnit4ClassRunner.java:57)\n"
                + "ParentRunner$3 run() 290 - (ParentRunner.java:290)\n"
                + "ParentRunner$1 schedule() 71 - (ParentRunner.java:71)\n"
                + "ParentRunner runChildren() 288 - (ParentRunner.java:288)\n"
                + "ParentRunner access$000() 58 - (ParentRunner.java:58)\n"
                + "ParentRunner$2 evaluate() 268 - (ParentRunner.java:268)\n"
                + "ParentRunner run() 363 - (ParentRunner.java:363)\n"
                + "JUnit4TestReference run() 89 - (JUnit4TestReference.java:89)\n"
                + "TestExecution run() 41 - (TestExecution.java:41)\n"
                + "RemoteTestRunner runTests() 541 - (RemoteTestRunner.java:541)\n"
                + "RemoteTestRunner runTests() 763 - (RemoteTestRunner.java:763)\n"
                + "RemoteTestRunner run() 463 - (RemoteTestRunner.java:463)\n"
                + "RemoteTestRunner main() 209 - (RemoteTestRunner.java:209)\n";

        assertEquals(expected, actual);
    }

}
