package de.duehl.basics.text;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;
import static de.duehl.basics.text.Text.LINE_BREAK;
import static de.duehl.basics.text.Text.LINE_BREAK_PLACE_HOLDER;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import org.junit.Test;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.basics.text.Text;
import de.duehl.basics.text.data.FoundSearch;

public class TextTest {

    @Test
    public void break001() {
        String text = "Mal sehen was so passiert! ";
        String expected = "Mal sehen was so passiert! ";
        int lineLength = 80;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break002() {
        String text = "Mal sehen was so passiert! ";
        String expected = "Mal sehen was" + LINE_BREAK + "so passiert! ";
        int lineLength = 15;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break003() {
        String text = "Mal sehen was so passiert!";
        String expected = "Mal" + LINE_BREAK
                + "sehen" + LINE_BREAK
                + "was" + LINE_BREAK
                + "so" + LINE_BREAK
                + "passiert!";
        int lineLength = 5;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break004() {
        String text = "Mal sehen was so AUSSERGEWÖHNLICHLANGES passiert!";
        String expected = "Mal" + LINE_BREAK
                + "sehen" + LINE_BREAK
                + "was" + LINE_BREAK
                + "so" + LINE_BREAK
                + "AUSSERGEWÖHNLICHLANGES" + LINE_BREAK
                + "passiert!";
        int lineLength = 5;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break005() {
        String text = "Mal_sehen_was_so_AUSSERGEWÖHNLICHLANGES_passiert!";
        String expected = text;
        int lineLength = 5;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break006() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 2;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break007() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 1;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break008() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 3;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break009() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 4;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break010() {
        String text = "ab cd ef";
        String expected = "ab cd" + LINE_BREAK + "ef";
        int lineLength = 5;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break011() {
        String text = "ab cd ef gh ij kl mn op qr";
        String expected = "ab cd" + LINE_BREAK + "ef gh" + LINE_BREAK + "ij kl"
                + LINE_BREAK + "mn op" + LINE_BREAK + "qr";
        int lineLength = 5;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void break012() {
        String text = "Mal\nsehen\r\nwas so AUSSERGEWÖHNLICHLANGES passiert! ";
        String expected = "Mal" + LINE_BREAK
                + "sehen" + LINE_BREAK
                + "was so" + LINE_BREAK
                + "AUSSERGEWÖHNLICHLANGES" + LINE_BREAK
                + "passiert! ";
        int lineLength = 10;
        String actual = Text.addLineBreaks(text, lineLength);
        assertEquals(expected, actual);
    }

    @Test
    public void countCharInString01() {
        String text = "foobarbazo";
        String search = "o";
        int expected = 3;
        int actual = Text.countPartInString(text, search);
        assertEquals(expected, actual);
    }

    @Test
    public void countCharInString02() {
        String text = "hahahahahahahahahaha";
        String search = "haha";
        int expected = 5;
        int actual = Text.countPartInString(text, search);
        assertEquals(expected, actual);
    }

    @Test
    public void countCharInString03() {
        String text = "10.234.642,12";
        String search = ".";
        int expected = 2;
        int actual = Text.countPartInString(text, search);
        assertEquals(expected, actual);
    }

    @Test
    public void countCharInStringPipes() {
        String text = "(Softwareentwicklung sowie IT-Consulting).|dmaic software GmbH & Co. KG";
        String search = "|";
        int expected = 1;
        int actual = Text.countPartInString(text, search);
        assertEquals(expected, actual);
    }

    @Test
    public void stripWhitespace01() {
        String text = "";
        String expected = "";
        String actual = Text.stripWhitespace(text);
        assertEquals(expected, actual);
    }

    @Test
    public void stripWhitespace02() {
        String text = " abc ";
        String expected = "abc";
        String actual = Text.stripWhitespace(text);
        assertEquals(expected, actual);
    }

    @Test
    public void stripWhitespace03() {
        String text = " \r\na     b\tc \n";
        String expected = "a b c";
        String actual = Text.stripWhitespace(text);
        assertEquals(expected, actual);
    }

    @Test
    public void firstCharsAreEqual01() {
        String text1 = "abcd";
        String text2 = "abcd";
        int numberOfChars = 5;
        assertTrue(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual02() {
        String text1 = "abcd";
        String text2 = "abcd";
        int numberOfChars = 4;
        assertTrue(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual03() {
        String text1 = "abcd";
        String text2 = "abcd";
        int numberOfChars = 3;
        assertTrue(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual04() {
        String text1 = "abcd";
        String text2 = "abcd";
        int numberOfChars = 1;
        assertTrue(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual05() {
        String text1 = "abcd";
        String text2 = "abcd";
        int numberOfChars = 0;
        assertTrue(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test (expected = IllegalArgumentException.class)
    public void firstCharsAreEqual06() {
        String text1 = "abcd";
        String text2 = "abcd";
        int numberOfChars = -1;
        Text.firstCharsAreEqual(text1, text2, numberOfChars);
    }

    @Test
    public void firstCharsAreEqual07() {
        String text1 = "abcd";
        String text2 = "abce";
        int numberOfChars = 3;
        assertTrue(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual08() {
        String text1 = "abcd";
        String text2 = "abee";
        int numberOfChars = 3;
        assertFalse(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual09() {
        String text1 = "abcd";
        String text2 = "aece";
        int numberOfChars = 3;
        assertFalse(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void firstCharsAreEqual10() {
        String text1 = "abcd";
        String text2 = "ebce";
        int numberOfChars = 3;
        assertFalse(Text.firstCharsAreEqual(text1, text2, numberOfChars));
    }

    @Test
    public void findAllPositions01() {
        String searchText = "foofoo";
        String text = "foofoofoo";
        List<Integer> indices = Text.findAllPositions(searchText, text);
        assertTrue(null != indices);
        assertTrue(2 == indices.size());
        assertEquals(0, (int) indices.get(0));
        assertEquals(3, (int) indices.get(1));
    }

    @Test
    public void findAllPositions02() {
        String searchText = "foo";
        String text = "foofoofoo";
        List<Integer> indices = Text.findAllPositions(searchText, text);
        assertTrue(null != indices);
        assertTrue(3 == indices.size());
        assertEquals(0, (int) indices.get(0));
        assertEquals(3, (int) indices.get(1));
        assertEquals(6, (int) indices.get(2));
    }

    @Test
    public void splitByWhitespaceEmptyInput() {
        String input = "";
        List<String> parts = Text.splitByWhitespace(input);
        assertEquals(0, parts.size());
    }

    @Test
    public void splitByWhitespace01() {
        String input = "foo";
        List<String> parts = Text.splitByWhitespace(input);
        assertTrue(1 == parts.size());
        assertEquals(input, parts.get(0));
    }

    @Test
    public void splitByWhitespace02() {
        String input = "foo bar";
        List<String> parts = Text.splitByWhitespace(input);
        assertTrue(2 == parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
    }

    @Test
    public void splitByWhitespace03() {
        String input = "foo bar       baz";
        List<String> parts = Text.splitByWhitespace(input);
        assertTrue(3 == parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void lineBreakHiding01() {
        String input = "a" + LINE_BREAK + "b" + LINE_BREAK + "c";
        String expected = "a" + LINE_BREAK_PLACE_HOLDER + "b" + LINE_BREAK_PLACE_HOLDER + "c";
        String actual = Text.hideLineBreaks(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lineBreakHiding02() {
        String input = "a" + LINE_BREAK_PLACE_HOLDER + "b" + LINE_BREAK_PLACE_HOLDER + "c";
        String expected = "a" + LINE_BREAK + "b" + LINE_BREAK + "c";
        String actual = Text.showLineBreaksAfterHiding(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lineBreaksToSpace01() {
        String input = "a" + LINE_BREAK + "b" + LINE_BREAK + "c";
        String expected = "a b c";
        String actual = Text.lineBreaksToSpace(input);
        assertEquals(expected, actual);
    }

    @Test
    public void removeLlineBreaksAndTabulators() {
        String input = "a" + LINE_BREAK + " b\t  c    " + LINE_BREAK + "d";
        String expected = "a b c d";
        String actual = Text.removeLineBreaksAndTabulators(input);
        assertEquals(expected, actual);
    }

    @Test
    public void removeLlineBreaksAndTabulatorsLineBreakAtEnd() {
        String input = "a" + LINE_BREAK;
        String expected = "a";
        String actual = Text.removeLineBreaksAndTabulators(input);
        assertEquals(expected, actual);
    }

    @Test
    public void stripContentsFromLines01() {
        String line1 = "#Dies";
        String line2 = "ist";
        String line3 = "# ein   ";
        String line4 = "Test";
        List<String> lines = new ArrayList<>();
        lines.add(line1);
        lines.add(line2);
        lines.add(line3);
        lines.add(line4);
        List<String> stripped = Text.stripCommentsFromLines(lines);
        assertEquals(2, stripped.size());
        assertEquals(line2, stripped.get(0));
        assertEquals(line4, stripped.get(1));
    }

    @Test
    public void stripContentsFromLines02() {
        String line1 = "#Dies";
        String line2 = "ist";
        String line3 = "# ein   ";
        String line4 = " # Test";
        List<String> lines = new ArrayList<>();
        lines.add(line1);
        lines.add(line2);
        lines.add(line3);
        lines.add(line4);
        List<String> stripped = Text.stripCommentsFromLines(lines);
        assertEquals(2, stripped.size());
        assertEquals(line2, stripped.get(0));
        assertEquals(line4, stripped.get(1));
    }

    @Test
    public void removeLastCharactersEmptyString() {
        String input = "";
        String actual = Text.removeLastCharacters(input, 5);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test (expected = NullPointerException.class)
    public void removeLastCharactersNullString() {
        String input = null;
        Text.removeLastCharacters(input, 5);
    }

    @Test
    public void removeLastCharactersFromTooSmallString() {
        String input = "abc";
        String actual = Text.removeLastCharacters(input, 5);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLastCharactersFromEqualLengthString() {
        String input = "abcde";
        String actual = Text.removeLastCharacters(input, 5);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLastCharactersFromLongerString() {
        String input = "abcdef";
        String actual = Text.removeLastCharacters(input, 5);
        String expected = "a";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLastCharactersToDeleteZeroChars() {
        String input = "abcdef";
        String actual = Text.removeLastCharacters(input, 0);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test (expected = StringIndexOutOfBoundsException.class)
    public void removeLastCharactersToDeleteANegativeNumberOfChars() {
        String input = "abcdef";
        String actual = Text.removeLastCharacters(input, -4);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesNormalCase() {
        String input = "abc";
        String actual = Text.fillWithSpaces(input, 5);
        String expected = "abc  ";
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesZeroNumber() {
        String input = "abc";
        String actual = Text.fillWithSpaces(input, 0);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesNegativeNumber() {
        String input = "abc";
        String actual = Text.fillWithSpaces(input, -7);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesNumberEqulasInputLength() {
        String input = "abc";
        String actual = Text.fillWithSpaces(input, input.length());
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesEmptyInput() {
        String input = "";
        String actual = Text.fillWithSpaces(input, 3);
        String expected = "   ";
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesAtFrontNormalCase() {
        String input = "abc";
        String actual = Text.fillWithSpacesAtFront(input, 5);
        String expected = "  abc";
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesAtFrontZeroNumber() {
        String input = "abc";
        String actual = Text.fillWithSpacesAtFront(input, 0);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesAtFrontNegativeNumber() {
        String input = "abc";
        String actual = Text.fillWithSpacesAtFront(input, -7);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesAtFrontNumberEqulasInputLength() {
        String input = "abc";
        String actual = Text.fillWithSpacesAtFront(input, input.length());
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithSpacesAtFrontEmptyInput() {
        String input = "";
        String actual = Text.fillWithSpacesAtFront(input, 3);
        String expected = "   ";
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithLinesNormalCase() {
        String input = "abc";
        String actual = Text.fillWithLines(input, 5);
        String expected = "abc--";
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithLinesZeroNumber() {
        String input = "abc";
        String actual = Text.fillWithLines(input, 0);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithLinesNegativeNumber() {
        String input = "abc";
        String actual = Text.fillWithLines(input, -7);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithLinesNumberEqulasInputLength() {
        String input = "abc";
        String actual = Text.fillWithLines(input, input.length());
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void fillWithLinesEmptyInput() {
        String input = "";
        String actual = Text.fillWithLines(input, 3);
        String expected = "---";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherWithEmptyList() {
        List<String> terms = new ArrayList<>();
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(terms.size(), cleanedTerms.size());
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherWithEmptySet() {
        Set<String> terms = new HashSet<>();
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(terms.size(), cleanedTerms.size());
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherTwoWordsContainingEachOther() {
        List<String> terms = new ArrayList<>();
        terms.add("Blume");
        terms.add("Blumenstrauß");
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(1, cleanedTerms.size());
        assertEquals("Blumenstrauß", cleanedTerms.get(0));
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherTwoSameWords() {
        List<String> terms = new ArrayList<>();
        terms.add("Blumenstrauß");
        terms.add("Blumenstrauß");
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(1, cleanedTerms.size());
        assertEquals("Blumenstrauß", cleanedTerms.get(0));
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherTwoWordsContainingEachOtherInternal() {
        List<String> terms = new ArrayList<>();
        String term1 = "Franz-Josef-Strauß Weg 17";
        String term2 = "Josef";
        terms.add(term1);
        terms.add(term2);
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(1, cleanedTerms.size());
        assertEquals(term1, cleanedTerms.get(0));
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherTwoWordsContainingEachOtherInternal2() {
        List<String> terms = new ArrayList<>();
        String term1 = "Josef";
        String term2 = "Franz-Josef-Strauß Weg 17";
        terms.add(term1);
        terms.add(term2);
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(1, cleanedTerms.size());
        assertEquals(term2, cleanedTerms.get(0));
    }

    @Test
    public void cleanCollectionOfStringsToNotContainingEachOtherThreeWordsContainingEachOther() {
        List<String> terms = new ArrayList<>();
        String term1 = "Franz-Josef-Strauß Weg 17";
        String term2 = "Josef-Strauß";
        String term3 = "Josef";
        terms.add(term1);
        terms.add(term2);
        terms.add(term3);
        List<String> cleanedTerms = Text.cleanCollectionOfStringsToNotContainingEachOther(terms);
        assertEquals(1, cleanedTerms.size());
        assertEquals(term1, cleanedTerms.get(0));
    }

    @Test
    public void getLastPartAfterDividerNotContainingDivider() {
        String input = "foo.bar.baz";
        String actual = Text.getLastPartAfterDivider(input, ";");
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void getLastPartAfterDividerNormalCase() {
        String input = "foo.bar.baz";
        String actual = Text.getLastPartAfterDivider(input, ".");
        String expected = "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void getLastPartAfterDividerNothingBehindDevider() {
        String input = "foo.bar.baz.";
        String actual = Text.getLastPartAfterDivider(input, ".");
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void joinThreeParts() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String delimiter = "#;#";

        String actual = Text.join(delimiter, parts);
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinEmptyList() {
        List<String> parts = new ArrayList<>();

        String delimiter = "#;#";

        String actual = Text.join(delimiter, parts);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithTabulator() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String actual = Text.joinWithTabulator(parts);
        String delimiter = "\t";
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithLineBreak() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String actual = Text.joinWithLineBreak(parts);
        String delimiter = LINE_BREAK;
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithLineBreaks2() {
        List<String> list = new ArrayList<>();
        list.add("foo");
        list.add("bar");
        list.add("baz");
        String actual = Text.joinWithLineBreak(list);
        String expected = "foo" + LINE_BREAK + "bar" + LINE_BREAK + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithPipe() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String actual = Text.joinWithPipe(parts);
        String delimiter = "|";
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinEptyListWithPipe() {
        List<String> parts = new ArrayList<>();

        String actual = Text.joinWithPipe(parts);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithBlank() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String actual = Text.joinWithBlank(parts);
        String delimiter = " ";
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithSemicolon() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String actual = Text.joinWithSemicolon(parts);
        String delimiter = ";";
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinWithColon() {
        List<String> parts = new ArrayList<>();
        parts.add("foo");
        parts.add("bar");
        parts.add("baz");

        String actual = Text.joinWithColon(parts);
        String delimiter = ":";
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinThreePartsViaEllipse() {
        String delimiter = "#;#";

        String actual = Text.join(delimiter, "foo", "bar", "baz");
        String expected = "foo" + delimiter + "bar" + delimiter + "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void joinNoPartsViaEllipse() {
        String delimiter = "#;#";

        String actual = Text.join(delimiter);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void splitByWhitespaceSize() {
        String input = "foo bar \t \n\rbaz";
        List<String> parts = Text.splitByWhitespace(input);
        assertEquals(3, parts.size());
    }

    @Test
    public void splitByWhitespace() {
        String input = "foo bar \t baz";
        List<String> parts = Text.splitByWhitespace(input);
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void splitByMultipleSpaces() {
        String input = "eins zwei  drei   vier    fünf";
        List<String> parts = Text.splitByMultipleSpaces(input, 3);
        assertEquals(3, parts.size());
        assertEquals("eins zwei  drei", parts.get(0));
        assertEquals("vier", parts.get(1));
        assertEquals("fünf", parts.get(2));
    }

    @Test (expected = IllegalArgumentException.class)
    public void splitByMultipleSpacesWithZeroSpaces() {
        String input = "eins zwei  drei   vier    fünf";
        Text.splitByMultipleSpaces(input, 0);
    }

    @Test
    public void splitBySize() {
        String input = "foo | bar";
        List<String> parts = Text.splitBy(input, "\\s*\\|\\s*");
        assertEquals(2, parts.size());
    }

    @Test
    public void splitBySize3() {
        String input = "foo | bar | baz";
        List<String> parts = Text.splitBy(input, "\\s*\\|\\s*");
        assertEquals(3, parts.size());
    }

    @Test
    public void splitBy() {
        String input = "foo | bar | baz";
        List<String> parts = Text.splitBy(input, "\\s*\\|\\s*");
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void splitByHashSize() {
        String input = "foo # bar";
        List<String> parts = Text.splitBy(input, "\\s*#\\s*");
        assertEquals(2, parts.size());
    }

    @Test
    public void splitByHashSize3() {
        String input = "foo # bar # baz";
        List<String> parts = Text.splitBy(input, "\\s*#\\s*");
        assertEquals(3, parts.size());
    }

    @Test
    public void splitByHash() {
        String input = "foo # bar # baz";
        List<String> parts = Text.splitBy(input, "\\s*#\\s*");
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void splitByNothingSize() {
        String input = "abc";
        List<String> parts = Text.splitBy(input, "");
        assertEquals(3, parts.size());
    }

    @Test
    public void splitByNothing() {
        String input = "abc";
        List<String> parts = Text.splitBy(input, "");
        assertEquals("a", parts.get(0));
        assertEquals("b", parts.get(1));
        assertEquals("c", parts.get(2));
    }

    @Test
    public void splitEmptyStringByNothingSize() {
        String input = "";
        List<String> parts = Text.splitBy(input, "");
        assertEquals(0, parts.size());
    }

    @Test
    public void splitByPipeSize() {
        String input = "foo | bar";
        List<String> parts = Text.splitByPipe(input);
        assertEquals(2, parts.size());
    }

    @Test
    public void splitByPipe() {
        String input = "foo | bar | baz";
        List<String> parts = Text.splitByPipe(input);
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void splitStringWithputPipeByPipeSize() {
        String input = "abc";
        List<String> parts = Text.splitByPipe(input);
        assertEquals(1, parts.size());
    }

    @Test
    public void splitStringWithputPipeByPipe() {
        String input = "foo";
        List<String> parts = Text.splitByPipe(input);
        assertEquals("foo", parts.get(0));
    }

    @Test
    public void splitEmptyStringByPipeSize() {
        String input = "";
        List<String> parts = Text.splitByPipe(input);
        assertEquals(0, parts.size());
    }

    @Test
    public void splitByPipeWithoutPipeSize() {
        String input = "foo bar";
        List<String> parts = Text.splitByPipe(input);
        assertEquals(1, parts.size());
    }

    @Test
    public void splitByPipeWithoutPipe() {
        String input = "foo bar";
        List<String> parts = Text.splitByPipe(input);
        assertEquals(input, parts.get(0));
    }

    @Test
    public void splitByPipeNotConsumingWhitespace() {
        String input = "foo | bar | baz";
        List<String> parts = Text.splitByPipeNotConsumingWhitespace(input);
        assertEquals("foo ", parts.get(0));
        assertEquals(" bar ", parts.get(1));
        assertEquals(" baz", parts.get(2));
    }

    @Test
    public void splitByQuotedBlankOrHashSize() {
        String input = "foo '#' bar ' ' baz baz2 # baz3";
        List<String> parts = Text.splitBy(input, "\\s*'(#| )'\\s*");
        assertEquals(3, parts.size());
    }

    @Test
    public void splitByQuotedBlankOrHash() {
        String input = "foo '#' bar ' ' baz baz2 # baz3";
        List<String> parts = Text.splitBy(input, "\\s*'(#| )'\\s*");
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz baz2 # baz3", parts.get(2));
    }

    @Test
    public void splitByQuotedBlankOrHashAsMethodSize() {
        String input = "foo '#' bar ' ' baz baz2 # baz3";
        List<String> parts = Text.splitByQuotedBlankOrHash(input);
        assertEquals(3, parts.size());
    }

    @Test
    public void splitByQuotedBlankOrHashAsMethod() {
        String input = "foo '#' bar ' ' baz baz2 # baz3";
        List<String> parts = Text.splitByQuotedBlankOrHash(input);
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz baz2 # baz3", parts.get(2));
    }

    @Test
    public void splitByAdditionSize() {
        String input = "foo + bar + baz baz2 # baz3";
        List<String> parts = Text.splitByAddition(input);
        assertEquals(3, parts.size());
    }

    @Test
    public void splitByAddition() {
        String input = "foo + bar + baz baz2 # baz3";
        List<String> parts = Text.splitByAddition(input);
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz baz2 # baz3", parts.get(2));
    }

    @Test
    public void splitBySubtractionSize() {
        String input = "foo - bar - baz baz2 # baz3";
        List<String> parts = Text.splitBySubtraction(input);
        assertEquals(3, parts.size());
    }

    @Test
    public void splitBySubtraction() {
        String input = "foo - bar - baz baz2 # baz3";
        List<String> parts = Text.splitBySubtraction(input);
        assertEquals(3, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz baz2 # baz3", parts.get(2));
    }

    @Test
    public void splitByKomma() {
        String input = "foo , bar, baz ,foo,bar";
        List<String> parts = Text.splitByKomma(input);
        assertEquals(5, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitByTabulator() {
        String input = "foo \t bar\t baz \tfoo\tbar";
        List<String> parts = Text.splitByTabulator(input);
        assertEquals(5, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitByTabulatorMultipleTabulators() {
        String input = "foo\t\tbar";
        List<String> parts = Text.splitByTabulator(input);
        assertEquals(3, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("", parts.get(1));
        assertEquals("bar", parts.get(2));
    }

    @Test
    public void splitByTabulatorWithNoTabulatorInInput() {
        String input = "foo bar baz";
        List<String> parts = Text.splitByTabulator(input);
        assertEquals(1, parts.size());
        assertEquals("foo bar baz", parts.get(0));
    }

    @Test
    public void splitByTabulatorEmptyInput() {
        String input = "";
        List<String> parts = Text.splitByTabulator(input);
        assertEquals(0, parts.size());
    }

    @Test
    public void splitByDot() {
        String input = "foo.bar.baz";
        List<String> parts = Text.splitByDot(input);
        assertEquals(3, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void splitByDotMultipleDots() {
        String input = "foo..baz";
        List<String> parts = Text.splitByDot(input);
        assertEquals(3, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("", parts.get(1));
        assertEquals("baz", parts.get(2));
    }

    @Test
    public void splitByTabulatorNotConsumingWhitespace() {
        String input = "foo \t bar\t baz \tfoo\tbar";
        List<String> parts = Text.splitByTabulatorNotConsumingWhitespace(input);
        assertEquals(5, parts.size());
        assertEquals("foo ", parts.get(0));
        assertEquals(" bar", parts.get(1));
        assertEquals(" baz ", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitBySemicolon() {
        String input = "foo ; bar; baz ;foo;bar";
        List<String> parts = Text.splitBySemicolon(input);
        assertEquals(5, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitBySemicolonWithNoSemikolonInInput() {
        String input = "foo bar baz";
        List<String> parts = Text.splitBySemicolon(input);
        assertEquals(1, parts.size());
        assertEquals("foo bar baz", parts.get(0));
    }

    @Test
    public void splitBySemicolonEmptyInput() {
        String input = "";
        List<String> parts = Text.splitBySemicolon(input);
        assertEquals(0, parts.size());
    }

    @Test
    public void splitBySemicolonNotConsumingWhitespace() {
        String input = "foo ; bar; baz ;foo;bar";
        List<String> parts = Text.splitBySemicolonNotConsumingWhitespace(input);
        assertEquals(5, parts.size());
        assertEquals("foo ", parts.get(0));
        assertEquals(" bar", parts.get(1));
        assertEquals(" baz ", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitByColon() {
        String input = "foo : bar: baz :foo:bar";
        List<String> parts = Text.splitByColon(input);
        assertEquals(5, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitByColonNotConsumingWhitespace() {
        String input = "foo : bar: baz :foo:bar";
        List<String> parts = Text.splitByColonNotConsumingWhitespace(input);
        assertEquals(5, parts.size());
        assertEquals("foo ", parts.get(0));
        assertEquals(" bar", parts.get(1));
        assertEquals(" baz ", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitBySpecial() {
        String input = "foo #;# bar#;# baz #;#foo#;#bar";
        List<String> parts = Text.splitBySpecial(input);
        assertEquals(5, parts.size());
        assertEquals("foo", parts.get(0));
        assertEquals("bar", parts.get(1));
        assertEquals("baz", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void splitBySpecialNotConsumingWhitespace() {
        String input = "foo #;# bar#;# baz #;#foo#;#bar";
        List<String> parts = Text.splitBySpecialNotConsumingWhitespace(input);
        assertEquals(5, parts.size());
        assertEquals("foo ", parts.get(0));
        assertEquals(" bar", parts.get(1));
        assertEquals(" baz ", parts.get(2));
        assertEquals("foo", parts.get(3));
        assertEquals("bar", parts.get(4));
    }

    @Test
    public void removeLineBreaksBlankInput() {
        String in = "";
        String actual = Text.removeLineBreaks(in);
        String expected = in;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreaksSingleSpace() {
        String in = " ";
        String actual = Text.removeLineBreaks(in);
        String expected = in;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreaksOneLineBreak() {
        String in = "foo" + Text.LINE_BREAK + "bar";
        String actual = Text.removeLineBreaks(in);
        String expected = "foo bar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreaksOneRN() {
        String in = "foo\r\nbar";
        String actual = Text.removeLineBreaks(in);
        String expected = "foo bar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreaksOneNR() {
        String in = "foo\n\rbar";
        String actual = Text.removeLineBreaks(in);
        String expected = "foo  bar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreaksOneN() {
        String in = "foo\nbar";
        String actual = Text.removeLineBreaks(in);
        String expected = "foo bar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreaksOneR() {
        String in = "foo\rbar";
        String actual = Text.removeLineBreaks(in);
        String expected = "foo bar";
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksBlankInput() {
        String in = "";
        String actual = Text.repairLineBreaks(in);
        String expected = in;
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksSingleSpace() {
        String in = " ";
        String actual = Text.repairLineBreaks(in);
        String expected = in;
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksOneLineBreak() {
        String in = "foo" + Text.LINE_BREAK + "bar";
        String actual = Text.repairLineBreaks(in);
        String expected = in;
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksOneRN() {
        String in = "foo\r\nbar";
        String actual = Text.repairLineBreaks(in);
        String expected = "foo" + Text.LINE_BREAK + "bar";
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksOneNR() {
        String in = "foo\n\rbar";
        String actual = Text.repairLineBreaks(in);
        String expected = "foo" + Text.LINE_BREAK + Text.LINE_BREAK + "bar";
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksOneN() {
        String in = "foo\nbar";
        String actual = Text.repairLineBreaks(in);
        String expected = "foo" + Text.LINE_BREAK + "bar";
        assertEquals(expected, actual);
    }

    @Test
    public void repairLineBreaksOneR() {
        String in = "foo\rbar";
        String actual = Text.repairLineBreaks(in);
        String expected = "foo" + Text.LINE_BREAK + "bar";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceBlankInput() {
        String in = "";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = in;
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceSingleSpace() {
        String in = " ";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceMultipleSpaces() {
        String in = "                   ";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceMultipleSpacesAroundChar() {
        String in = "             a      ";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "a";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceMultipleSpacesFront() {
        String in = "             a";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "a";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceMultipleSpacesRear() {
        String in = "a            ";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "a";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceMultipleSpacesBetween() {
        String in = "a            b";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "a b";
        assertEquals(expected, actual);
    }

    @Test
    public void trimAndCompactWhitespaceTwoSpacesBetween() {
        String in = "a  b";
        String actual = Text.trimAndCompactWhitespace(in);
        String expected = "a b";
        assertEquals(expected, actual);
    }

    @Test
    public void listOfLinesToString() {
        List<String> lines = new ArrayList<>();
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        String actual = Text.listOfLinesToString(lines);
        String expected = "foo" + Text.LINE_BREAK
                + "bar" + Text.LINE_BREAK
                + "baz" + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void stringToListOfLines() {
        String text = "foo" + Text.LINE_BREAK
                + "bar" + Text.LINE_BREAK
                + "baz" + Text.LINE_BREAK;
        List<String> lines = Text.stringToListOfLines(text);
        assertEquals(3, lines.size());
        assertEquals("foo", lines.get(0));
        assertEquals("bar", lines.get(1));
        assertEquals("baz", lines.get(2));
    }

    @Test
    public void stringToListOfLinesWithoutTrailingLineBreak() {
        String text = "foo" + Text.LINE_BREAK
                + "bar" + Text.LINE_BREAK
                + "baz";
        List<String> lines = Text.stringToListOfLines(text);
        assertEquals(3, lines.size());
        assertEquals("foo", lines.get(0));
        assertEquals("bar", lines.get(1));
        assertEquals("baz", lines.get(2));
    }

    @Test
    public void stringToListOfLinesEmptyText() {
        String text = "";
        List<String> lines = Text.stringToListOfLines(text);
        assertEquals(0, lines.size());
    }

    @Test
    public void stringToListOfLinesSingleLineBreak() {
        String text = Text.LINE_BREAK;
        List<String> lines = Text.stringToListOfLines(text);
        assertEquals(1, lines.size());
        assertTrue(lines.get(0).isEmpty());
    }

    @Test
    public void listOfLinesToStringAndViceVersa() {
        List<String> lines1 = new ArrayList<>();
        lines1.add("foo");
        lines1.add("bar");
        lines1.add("baz");
        String text1 = Text.listOfLinesToString(lines1);
        List<String> lines2 = Text.stringToListOfLines(text1);
        String text2 = Text.listOfLinesToString(lines2);
        assertEquals(text1, text2);
        assertEquals(lines1.size(), lines2.size());
        assertEquals(3, lines2.size());
        assertEquals(lines1.get(0), lines2.get(0));
        assertEquals(lines1.get(1), lines2.get(1));
        assertEquals(lines1.get(2), lines2.get(2));
    }

    @Test
    public void searchFirstLineIndexContainig1() {
        List<String> lines = new ArrayList<>();
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        String search = "foo";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void searchFirstLineIndexContainig2() {
        List<String> lines = new ArrayList<>();
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        String search = "bar";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test
    public void searchFirstLineIndexContainig3() {
        List<String> lines = new ArrayList<>();
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        String search = "baz";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = 2;
        assertEquals(expected, actual);
    }

    @Test
    public void searchFirstLineIndexContainigNotContainedSearch() {
        List<String> lines = new ArrayList<>();
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        String search = "hugo";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = Text.NOT_FOUND_INDEX;
        assertEquals(expected, actual);
    }

    @Test
    public void searchFirstLineIndexContainigEmptyList() {
        List<String> lines = new ArrayList<>();
        String search = "hugo";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = Text.NOT_FOUND_INDEX;
        assertEquals(expected, actual);
    }

    @Test
    public void searchFirstLineIndexContainigEmptySearch() {
        List<String> lines = new ArrayList<>();
        lines.add("foo");
        lines.add("bar");
        lines.add("baz");
        String search = "";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void searchFirstLineIndexContainigEmptyListAndEmptySearch() {
        List<String> lines = new ArrayList<>();
        String search = "";
        int actual = Text.searchFirstLineIndexContainig(lines, search);
        int expected = Text.NOT_FOUND_INDEX;
        assertEquals(expected, actual);
    }

    @Test
    public void removeFromTextEmptyString() {
        String text = "";
        String actual = Text.removeFromText(text, 0, 0);
        String expected = " ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeFromTextInTheMiddle() {
        String text = "abc";
        String actual = Text.removeFromText(text, 1, 1);
        String expected = "a c";
        assertEquals(expected, actual);
    }

    @Test
    public void removeFromTextFront() {
        String text = "abc";
        String actual = Text.removeFromText(text, 0, 1);
        String expected = " bc";
        assertEquals(expected, actual);
    }

    @Test
    public void removeFromTextRear() {
        String text = "abc";
        String actual = Text.removeFromText(text, 2, 1);
        String expected = "ab ";
        assertEquals(expected, actual);
    }

    @Test (expected = IllegalArgumentException.class)
    public void removeFromTextPositionTooSmall() {
        String text = "abc";
        Text.removeFromText(text, -1, 1);
    }

    @Test (expected = IllegalArgumentException.class)
    public void removeFromTextPositionTooBig() {
        String text = "abc";
        Text.removeFromText(text, 3, 1);
    }

    @Test (expected = IllegalArgumentException.class)
    public void removeFromTextLengthTooSmall() {
        String text = "abc";
        Text.removeFromText(text, 0, -2);
    }

    @Test (expected = IllegalArgumentException.class)
    public void removeFromTextLengthAndPositionTooBig() {
        String text = "abc";
        Text.removeFromText(text, 2, 2);
    }

    @Test
    public void replaceInTextMiddle() {
        String text = "abc";
        String replace = "###";
        int start = 1;
        int end = 2;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "a###c";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextMiddleZeroLength() {
        String text = "abc";
        String replace = "###";
        int start = 1;
        int end = 1;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "a###bc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextFront() {
        String text = "abc";
        String replace = "###";
        int start = 0;
        int end = 1;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "###bc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextFrontZeroLength() {
        String text = "abc";
        String replace = "###";
        int start = 0;
        int end = 0;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "###abc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextRear() {
        String text = "abc";
        String replace = "###";
        int start = 2;
        int end = 3;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "ab###";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextRearZeroLength() {
        String text = "abc";
        String replace = "###";
        int start = 2;
        int end = 2;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "ab###c";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextRearZeroLengthTotalRear() {
        String text = "abc";
        String replace = "###";
        int start = 3;
        int end = 3;
        String actual = Text.replaceInText(text, replace, start, end);
        String expected = "abc###";
        assertEquals(expected, actual);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextStartBiggerThanEnd() {
        String text = "abc";
        String replace = "###";
        int start = 2;
        int end = 1;
        Text.replaceInText(text, replace, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextStartTooSmall() {
        String text = "abc";
        String replace = "###";
        int start = -1;
        int end = 1;
        Text.replaceInText(text, replace, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextStartTooBig() {
        String text = "abc";
        String replace = "###";
        int start = 5;
        int end = 5;
        Text.replaceInText(text, replace, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextEndTooSmall() {
        String text = "abc";
        String replace = "###";
        int start = 2;
        int end = -1;
        Text.replaceInText(text, replace, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextEndTooBig() {
        String text = "abc";
        String replace = "###";
        int start = 2;
        int end = 4;
        Text.replaceInText(text, replace, start, end);
    }

    // TODO replaceInTextWithEqualLengthCharacterSequence(String text, char character,
    //      int start, int end)  auch Grenzen, end < start, end == start, end > length, start < 0 ...

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceMiddle() {
        String text = "abc";
        char character = '#';
        int start = 1;
        int end = 2;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "a#c";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceMiddleZeroLength() {
        String text = "abc";
        char character = '#';
        int start = 1;
        int end = 1;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "abc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceFront() {
        String text = "abc";
        char character = '#';
        int start = 0;
        int end = 1;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "#bc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceFrontZeroLength() {
        String text = "abc";
        char character = '#';
        int start = 0;
        int end = 0;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "abc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceRear() {
        String text = "abc";
        char character = '#';
        int start = 2;
        int end = 3;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "ab#";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceRearZeroLength() {
        String text = "abc";
        char character = '#';
        int start = 2;
        int end = 2;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "abc";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceRearZeroLengthTotalRear() {
        String text = "abc";
        char character = '#';
        int start = 3;
        int end = 3;
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
        String expected = "abc";
        assertEquals(expected, actual);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextWithEqualLengthCharacterSequenceStartBiggerThanEnd() {
        String text = "abc";
        char character = '#';
        int start = 2;
        int end = 1;
        Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextWithEqualLengthCharacterSequenceStartTooSmall() {
        String text = "abc";
        char character = '#';
        int start = -1;
        int end = 1;
        Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextWithEqualLengthCharacterSequenceStartTooBig() {
        String text = "abc";
        char character = '#';
        int start = 5;
        int end = 5;
        Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextWithEqualLengthCharacterSequenceEndTooSmall() {
        String text = "abc";
        char character = '#';
        int start = 2;
        int end = -1;
        Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
    }

    @Test (expected = IllegalArgumentException.class)
    public void replaceInTextWithEqualLengthCharacterSequenceEndTooBig() {
        String text = "abc";
        char character = '#';
        int start = 2;
        int end = 4;
        Text.replaceInTextWithEqualLengthCharacterSequence(text, character, start, end);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceOnce() {
        String text = "foo bar baz foo foo bar baz foo";
        String actual = Text.replaceInTextWithEqualLengthCharacterSequenceOnce(text, "bar");
        String expected = "foo ### baz foo foo bar baz foo";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceWithList1() {
        String text = "foo bar baz foo foo bar baz foo";
        List<String> replaceWords = CollectionsHelper.buildListFrom("bar");
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, replaceWords);
        String expected = "foo ### baz foo foo ### baz foo";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceWithList2() {
        String text = "foo bar baz foo foo bar baz foo";
        List<String> replaceWords = CollectionsHelper.buildListFrom("bar", "baz");
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, replaceWords);
        String expected = "foo ### ### foo foo ### ### foo";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceWithList3() {
        String text = "foo bar baz foo foo bar baz foo";
        List<String> replaceWords = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, replaceWords);
        String expected = "### ### ### ### ### ### ### ###";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceWithEmptyList() {
        String text = "foo bar baz foo foo bar baz foo";
        List<String> replaceWords = CollectionsHelper.buildListFrom();
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, replaceWords);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceWithListEmptyText() {
        String text = "";
        List<String> replaceWords = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, replaceWords);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void replaceInTextWithEqualLengthCharacterSequenceWithEmptyListAndEmptyText() {
        String text = "";
        List<String> replaceWords = CollectionsHelper.buildListFrom();
        String actual = Text.replaceInTextWithEqualLengthCharacterSequence(text, replaceWords);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void findInText01() {
        String found = Text.findInText("<id>15</id>", "<id>", "</id>");
        assertEquals("15", found);
    }

    @Test (expected = RuntimeException.class)
    public void findInText02() {
        Text.findInText("<id>15</od>", "<id>", "</id>");
    }

    @Test (expected = RuntimeException.class)
    public void findInText03() {
        Text.findInText("<id>15<id>", "<id>", "</id>");
    }

    @Test
    public void findInText04() {
        String found = Text.findInText("<foo><bar>BAR</bar><id>15</id></foo>", "<id>", "</id>");
        assertEquals("15", found);
    }

    @Test (expected = RuntimeException.class)
    public void findInTextMultipleStart() {
        Text.findInText("<id>15</id><id>", "<id>", "</id>");
    }

    @Test (expected = RuntimeException.class)
    public void findInTextMultipleEnd() {
        Text.findInText("<id>15</id></id>", "<id>", "</id>");
    }

    @Test (expected = RuntimeException.class)
    public void findInTextMissingStart() {
        Text.findInText("</id>15</id>", "<id>", "</id>");
    }

    @Test (expected = RuntimeException.class)
    public void findInTextMissingEnd() {
        Text.findInText("<id>15<id>", "<id>", "</id>");
    }

    @Test
    public void hideLineBreaks() {
        String input = "ein " + Text.LINE_BREAK + " Test";
        String actual = Text.hideLineBreaks(input);
        String expected = "ein " + Text.LINE_BREAK_PLACE_HOLDER + " Test";
        assertEquals(expected, actual);
    }

    @Test
    public void hideLineBreaksN() {
        String input = "ein \n Test";
        String actual = Text.hideLineBreaks(input);
        String expected = "ein " + Text.LINE_BREAK_PLACE_HOLDER + " Test";
        assertEquals(expected, actual);
    }

    @Test
    public void hideLineBreaksRN() {
        String input = "ein \r\n Test";
        String actual = Text.hideLineBreaks(input);
        String expected = "ein " + Text.LINE_BREAK_PLACE_HOLDER + " Test";
        assertEquals(expected, actual);
    }

    @Test
    public void hideLineBreaksWithLiteralBackslashN() {
        String input = "ein " + Text.LINE_BREAK + " Test";
        String actual = Text.hideLineBreaksWithLiteralBackslashN(input);
        String expected = "ein " + "\\n" + " Test";
        assertEquals(expected, actual);
    }

    @Test
    public void hideLineBreaksWithLiteralBackslashNN() {
        String input = "ein \n Test";
        String actual = Text.hideLineBreaksWithLiteralBackslashN(input);
        String expected = "ein " + "\\n" + " Test";
        assertEquals(expected, actual);
    }

    @Test
    public void hideLineBreaksWithLiteralBackslashNRN() {
        String input = "ein \r\n Test";
        String actual = Text.hideLineBreaksWithLiteralBackslashN(input);
        String expected = "ein " + "\\n" + " Test";
        assertEquals(expected, actual);
    }

    @Test
    public void doubleTwoDecimalPlacesWithTwoDecimalPlaces() {
        double input = 1.02d;
        String actual = Text.doubleTwoDecimalPlaces(input);
        String expected = "1,02";
        assertEquals(expected, actual);
    }

    @Test
    public void doubleTwoDecimalPlacesWithOneDecimalPlaces() {
        double input = 1.2d;
        String actual = Text.doubleTwoDecimalPlaces(input);
        String expected = "1,20";
        assertEquals(expected, actual);
    }

    @Test
    public void doubleTwoDecimalPlacesNotRounded() {
        double input = 1.234d;
        String actual = Text.doubleTwoDecimalPlaces(input);
        String expected = "1,23";
        assertEquals(expected, actual);
    }

    @Test
    public void doubleTwoDecimalPlacesRounded() {
        double input = 1.235d;
        String actual = Text.doubleTwoDecimalPlaces(input);
        String expected = "1,24";
        assertEquals(expected, actual);
    }

    @Test
    public void splitByLineBreaks() {
        String input = "foo" + LINE_BREAK + "bar" + LINE_BREAK + "baz";
        List<String> list = Text.splitByLineBreaks(input);
        assertEquals(3, list.size());
        assertEquals("foo", list.get(0));
        assertEquals("bar", list.get(1));
        assertEquals("baz", list.get(2));
    }

    @Test
    public void buildCompareText0() {
        String input = "Dies ist ein Probetext mit äöüß Umlauten ÄÖÜ.";
        String actual = Text.buildCompareText(input);
        String expected = "dies ist ein probetext mit aeoeuess umlauten aeoeue.";
        assertEquals(expected, actual);
    }

    @Test
    public void buildCompareText1() {
        String input = "Blaubären schießen über Vögel.";
        String actual = Text.buildCompareText(input);
        assertEquals("blaubaeren schiessen ueber voegel.", actual);
    }

    @Test
    public void buildCompareText2() {
        String input = "ÄäÖöÜüSSßfooäÖäÖäü.,-#|";
        String actual = Text.buildCompareText(input);
        assertEquals("aeaeoeoeueuessssfooaeoeaeoeaeue.,-#|", actual);
    }

    @Test
    public void isWordChar() {
        assertTrue(Text.isWordChar('a'));
        assertTrue(Text.isWordChar('B'));
        assertTrue(Text.isWordChar('ä'));
        assertTrue(Text.isWordChar('#'));

        assertFalse(Text.isWordChar(' '));
        assertFalse(Text.isWordChar('\n'));
        assertFalse(Text.isWordChar('\t'));

        assertFalse(Text.isWordChar('-'));
        assertFalse(Text.isWordChar(','));

        assertFalse(Text.isWordChar('('));
        assertFalse(Text.isWordChar(')'));
        assertFalse(Text.isWordChar('['));
        assertFalse(Text.isWordChar(']'));
        assertFalse(Text.isWordChar('{'));
        assertFalse(Text.isWordChar('}'));
    }

    @Test
    public void isWhitespace() {
        assertTrue(Text.isWhitespace(' '));
        assertTrue(Text.isWhitespace('\n'));
        assertTrue(Text.isWhitespace('\t'));

        assertFalse(Text.isWhitespace('a'));
        assertFalse(Text.isWhitespace('B'));

        assertFalse(Text.isWhitespace('-'));
        assertFalse(Text.isWhitespace(','));

        assertFalse(Text.isWhitespace('('));
        assertFalse(Text.isWhitespace(')'));
        assertFalse(Text.isWhitespace('['));
        assertFalse(Text.isWhitespace(']'));
        assertFalse(Text.isWhitespace('{'));
        assertFalse(Text.isWhitespace('}'));
    }

    @Test
    public void isBrace() {
        assertTrue(Text.isBrace('('));
        assertTrue(Text.isBrace(')'));
        assertTrue(Text.isBrace('['));
        assertTrue(Text.isBrace(']'));
        assertTrue(Text.isBrace('{'));
        assertTrue(Text.isBrace('}'));

        assertFalse(Text.isBrace(' '));
        assertFalse(Text.isBrace('\n'));
        assertFalse(Text.isBrace('\t'));

        assertFalse(Text.isBrace('a'));
        assertFalse(Text.isBrace('B'));

        assertFalse(Text.isBrace('-'));
        assertFalse(Text.isBrace(','));
    }

    @Test
    public void isPunctuation() {
        assertTrue(Text.isPunctuation('-'));
        assertTrue(Text.isPunctuation('_'));
        assertTrue(Text.isPunctuation(','));
        assertTrue(Text.isPunctuation('.'));
        assertTrue(Text.isPunctuation(':'));
        assertTrue(Text.isPunctuation(';'));
        assertTrue(Text.isPunctuation('!'));
        assertTrue(Text.isPunctuation('?'));

        assertFalse(Text.isPunctuation('('));
        assertFalse(Text.isPunctuation(')'));
        assertFalse(Text.isPunctuation('['));
        assertFalse(Text.isPunctuation(']'));
        assertFalse(Text.isPunctuation('{'));
        assertFalse(Text.isPunctuation('}'));

        assertFalse(Text.isPunctuation(' '));
        assertFalse(Text.isPunctuation('\n'));
        assertFalse(Text.isPunctuation('\t'));

        assertFalse(Text.isPunctuation('a'));
        assertFalse(Text.isPunctuation('B'));
    }

    @Test
    public void isCapitalLetter() {
        assertTrue(Text.isCapitalLetter('A'));
        assertTrue(Text.isCapitalLetter('B'));
        assertTrue(Text.isCapitalLetter('Ä'));
        assertTrue(Text.isCapitalLetter('Ö'));
        assertTrue(Text.isCapitalLetter('Ü'));

        assertFalse(Text.isCapitalLetter('a'));
        assertFalse(Text.isCapitalLetter('ä'));
        assertFalse(Text.isCapitalLetter('ö'));
        assertFalse(Text.isCapitalLetter('ü'));
        assertFalse(Text.isCapitalLetter('ß'));

        assertFalse(Text.isCapitalLetter('('));
        assertFalse(Text.isCapitalLetter(')'));
        assertFalse(Text.isCapitalLetter('['));
        assertFalse(Text.isCapitalLetter(']'));
        assertFalse(Text.isCapitalLetter('{'));
        assertFalse(Text.isCapitalLetter('}'));

        assertFalse(Text.isCapitalLetter(' '));
        assertFalse(Text.isCapitalLetter('\n'));
        assertFalse(Text.isCapitalLetter('\t'));

        assertFalse(Text.isCapitalLetter('-'));
        assertFalse(Text.isCapitalLetter('_'));
        assertFalse(Text.isCapitalLetter(','));
        assertFalse(Text.isCapitalLetter('.'));
        assertFalse(Text.isCapitalLetter(':'));
        assertFalse(Text.isCapitalLetter(';'));
        assertFalse(Text.isCapitalLetter('!'));
        assertFalse(Text.isCapitalLetter('?'));
    }

    @Test
    public void firstLetterIsCapitalLetter() {
        assertTrue(Text.firstLetterIsCapitalLetter("Aber"));
        assertTrue(Text.firstLetterIsCapitalLetter("Bärbel"));
        assertTrue(Text.firstLetterIsCapitalLetter("Cäsar"));
        assertTrue(Text.firstLetterIsCapitalLetter("David"));
        assertTrue(Text.firstLetterIsCapitalLetter("Edih"));
        assertTrue(Text.firstLetterIsCapitalLetter("Fahrrad"));
        assertTrue(Text.firstLetterIsCapitalLetter("Gerd"));
        assertTrue(Text.firstLetterIsCapitalLetter("Haus"));
        assertTrue(Text.firstLetterIsCapitalLetter("Igel"));
        assertTrue(Text.firstLetterIsCapitalLetter("Jörg"));
        assertTrue(Text.firstLetterIsCapitalLetter("Karl"));
        assertTrue(Text.firstLetterIsCapitalLetter("Libelle"));
        assertTrue(Text.firstLetterIsCapitalLetter("Marmelade"));
        assertTrue(Text.firstLetterIsCapitalLetter("Notfallverordnung"));
        assertTrue(Text.firstLetterIsCapitalLetter("Oregano"));
        assertTrue(Text.firstLetterIsCapitalLetter("Pausenbrot"));
        assertTrue(Text.firstLetterIsCapitalLetter("Quelle"));
        assertTrue(Text.firstLetterIsCapitalLetter("Rand"));
        assertTrue(Text.firstLetterIsCapitalLetter("Sand"));
        assertTrue(Text.firstLetterIsCapitalLetter("Tand"));
        assertTrue(Text.firstLetterIsCapitalLetter("Urmel"));
        assertTrue(Text.firstLetterIsCapitalLetter("Ventil"));
        assertTrue(Text.firstLetterIsCapitalLetter("Wolke"));
        assertTrue(Text.firstLetterIsCapitalLetter("Xantippe"));
        assertTrue(Text.firstLetterIsCapitalLetter("Ypsilon"));
        assertTrue(Text.firstLetterIsCapitalLetter("Zahl"));
        assertTrue(Text.firstLetterIsCapitalLetter("Ärmel"));
        assertTrue(Text.firstLetterIsCapitalLetter("Öl"));
        assertTrue(Text.firstLetterIsCapitalLetter("Übertrag"));

        assertFalse(Text.firstLetterIsCapitalLetter("aber"));
        assertFalse(Text.firstLetterIsCapitalLetter("ärmel"));
        assertFalse(Text.firstLetterIsCapitalLetter("öl"));
        assertFalse(Text.firstLetterIsCapitalLetter("übertrag"));

        assertFalse(Text.firstLetterIsCapitalLetter(""));
    }

    @Test
    public void multipleString() {
        String text = "foo";
        int length = 3;
        String actual = Text.multipleString(text, length);
        String expected = "foofoofoo";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringSingleCharAsString() {
        String text = "A";
        int length = 7;
        String actual = Text.multipleString(text, length);
        String expected = "AAAAAAA";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringZeroLength() {
        String text = "foo bar baz";
        int length = 0;
        String actual = Text.multipleString(text, length);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringNegativeLength() {
        String text = "foo bar baz";
        int length = -15;
        String actual = Text.multipleString(text, length);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringZeroTextLength() {
        String text = "";
        int length = 250;
        String actual = Text.multipleString(text, length);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringCharacter() {
        char character = '#';
        int length = 3;
        String actual = Text.multipleString(character, length);
        String expected = "###";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringCharacterZeroLength() {
        char character = '#';
        int length = 0;
        String actual = Text.multipleString(character, length);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void multipleStringCharacterNegativeLength() {
        char character = '#';
        int length = -44;
        String actual = Text.multipleString(character, length);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void findIntersectionOfEmptyTexts() {
        String text1 = "";
        String text2 = "";
        String actual = Text.findIntersection(text1, text2);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void findIntersectionOfOneCharacterEqualTexts() {
        String text1 = "a";
        String text2 = "a";
        String actual = Text.findIntersection(text1, text2);
        String expected = "a";
        assertEquals(expected, actual);
    }

    @Test
    public void findIntersectionOneCharacter() {
        String text1 = "abc";
        String text2 = "cde";
        String actual = Text.findIntersection(text1, text2);
        String expected = "c";
        assertEquals(expected, actual);
    }

    @Test
    public void findIntersectionMetalle() {
        String text1 = "NE-Metalle";
        String text2 = "Metallen";
        String actual = Text.findIntersection(text1, text2);
        String expected = "Metalle";
        assertEquals(expected, actual);
    }

    @Test
    public void findIntersectionFails1() {
        String text1 = "NE-MetalleX";
        String text2 = "Metallen";
        String actual = Text.findIntersection(text1, text2);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void findIntersectionFails2() {
        String text1 = "NE-Metalle";
        String text2 = "XMetallen";
        String actual = Text.findIntersection(text1, text2);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void containsUpperLetter1() {
        String input = "foo bar baz";
        boolean actual = Text.containsUpperLetter(input);
        boolean expected = false;
        assertEquals(expected, actual);
    }

    @Test
    public void containsUpperLetter2() {
        String input = "foo Bar baz";
        boolean actual = Text.containsUpperLetter(input);
        boolean expected = true;
        assertEquals(expected, actual);
    }

    @Test
    public void containsUpperLetter3() {
        String input = "foo bar Übel baz";
        boolean actual = Text.containsUpperLetter(input);
        boolean expected = true;
        assertEquals(expected, actual);
    }

    @Test
    public void contains() {
        String input = "Der Gegenstand wird erweitert: Bla bla";
        List<String> searches = CollectionsHelper.buildListFrom("änder", "Änder", "aender",
                "Aender", "bericht", "Bericht", "erweiter", "Erweiter");
        boolean actual = Text.contains(input, searches);
        boolean expected = true;
        assertEquals(expected, actual);
    }

    @Test
    public void containsFailing() {
        String input = "Gegenstand: Bla bla";
        List<String> searches = CollectionsHelper.buildListFrom("änder", "Änder", "aender",
                "Aender", "bericht", "Bericht", "erweiter", "Erweiter");
        boolean actual = Text.contains(input, searches);
        boolean expected = false;
        assertEquals(expected, actual);
    }

    @Test
    public void toUpperCaseEmptyString() {
        String input = "";
        String actual = Text.toFirstUpperCase(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void toUpperCaseOneUpperLetter() {
        String input = "A";
        String actual = Text.toFirstUpperCase(input);
        String expected = "A";
        assertEquals(expected, actual);
    }

    @Test
    public void toUpperCaseOneLowerLetter() {
        String input = "a";
        String actual = Text.toFirstUpperCase(input);
        String expected = "A";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase01() {
        String input = "Hammer";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Hammer";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase02() {
        String input = "hammer";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Hammer";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase03() {
        String input = "hammer nagel Schere Licht!";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Hammer nagel schere licht!";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase04() {
        String input = "straße";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Straße";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase05() {
        String input = "ärger";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Ärger";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase06() {
        String input = "Übel";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Übel";
        assertEquals(expected, actual);
    }

    @Test
    public void toFirstUpperCase07() {
        String input = "BÖSE";
        String actual = Text.toFirstUpperCase(input);
        String expected = "Böse";
        assertEquals(expected, actual);
    }

    @Test
    public void javaToLowerCaseGerman() {
        String input = "BÖSE";
        String actual = input.toLowerCase(Locale.GERMAN);
        String expected = "böse";
        assertEquals(expected, actual);
    }

    @Test
    public void javaToLowerCaseGermanStrasse() {
        String input = "Straße";
        String actual = input.toLowerCase(Locale.GERMAN);
        String expected = "straße";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCaseEmptyString() {
        String input = "";
        String actual = Text.toCamelCase(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCaseOneUpperLetter() {
        String input = "A";
        String actual = Text.toCamelCase(input);
        String expected = "A";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCaseOneLowerLetter() {
        String input = "a";
        String actual = Text.toCamelCase(input);
        String expected = "A";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCase01() {
        String input = "Hammer";
        String actual = Text.toCamelCase(input);
        String expected = "Hammer";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCase02() {
        String input = "hammer";
        String actual = Text.toCamelCase(input);
        String expected = "Hammer";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCase03() {
        String input = "hammer nagel Schere Licht!";
        String actual = Text.toCamelCase(input);
        String expected = "HammerNagelSchereLicht!";
        assertEquals(expected, actual);
    }

    @Test
    public void toCamelCase04() {
        String input = "wort1 dieseWortHatBereitsCamelCase wort2";
        String actual = Text.toCamelCase(input);
        String expected = "Wort1DieseWortHatBereitsCamelCaseWort2";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameEmtyInput() {
        String input = "";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameUntrimmedInput() {
        String input = "    foo      \t  ";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameReplaceOneSpaceToOneUnderscore() {
        String input = "foo bar";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "foo_bar";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameReplaceMultipleSpaceToOneUnderscore() {
        String input = "foo       bar";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "foo_bar";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameReplaceMultipleUnderscoreSpaceMixToOneUnderscore() {
        String input = "foo  __   ___ _ _      _bar";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "foo_bar";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameReplaceUmlaute1() {
        String input = "Ärger Övelgönne Übel";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "Aerger_Oevelgoenne_Uebel";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameReplaceUmlaute2() {
        String input = "Krähe Kübel Straße";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "Kraehe_Kuebel_Strasse";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanForSasVariableNameReplaceRubbish() {
        String input = "foo -!,;.:-/\\0123456789 bar";
        String actual = Text.cleanForSasVariableName(input);
        String expected = "foo_bar";
        assertEquals(expected, actual);
    }

    @Test
    public void nicePath01() {
        String input = "     ";
        String actual = Text.nicePath(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void nicePath02() {
        String input = "c:\\temp";
        String actual = Text.nicePath(input);
        String expected = "c:\\temp";
        assertEquals(expected, actual);
    }

    @Test
    public void nicePath03() {
        String input = "c:/temp";
        String actual = Text.nicePath(input);
        String expected = "c:\\temp";
        assertEquals(expected, actual);
    }

    @Test
    public void nicePathes01() {
        List<String> list = CollectionsHelper.buildListFrom("     ", "c:\\temp", "c:/temp");
        List<String> actualList = Text.nicePathes(list);
        List<String> expectedList = CollectionsHelper.buildListFrom("", "c:\\temp", "c:\\temp");
        assertEquals(3, actualList.size());
        for (int index = 0; index < 3; ++index) {
            String actual = actualList.get(index);
            String expected = expectedList.get(index);
            assertEquals(expected, actual);
        }
    }

    @Test
    public void skipSublinkPartWithoutHashPart() {
        String input = "hilfe.html";
        String actual = Text.skipSublinkPart(input);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void skipSublinkPartWithHashPart() {
        String input = "hilfe.html#why";
        String actual = Text.skipSublinkPart(input);
        String expected = "hilfe.html";
        assertEquals(expected, actual);
    }

    @Test
    public void skipSublinkPartEmptyInput() {
        String input = "";
        String actual = Text.skipSublinkPart(input);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void skipSublinkPartEmptyInputWithHash() {
        String input = "#why";
        String actual = Text.skipSublinkPart(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void containsOnlyNormalLettersOnlyLowercaseLetters() {
        String input = "why";
        assertTrue(Text.containsOnlyNormalLetters(input));
    }

    @Test
    public void containsOnlyNormalLettersOnlyUppercaseLetters() {
        String input = "WHY";
        assertTrue(Text.containsOnlyNormalLetters(input));
    }

    @Test
    public void containsOnlyNormalLettersMixedLetters() {
        String input = "WHYareYOUsoSERIOUS";
        assertTrue(Text.containsOnlyNormalLetters(input));
    }

    @Test
    public void containsOnlyNormalLettersFailesUmlaut() {
        String input = "hähä";
        assertFalse(Text.containsOnlyNormalLetters(input));
    }

    @Test
    public void containsOnlyNormalLettersFailesSpace() {
        String input = "foo bar baz";
        assertFalse(Text.containsOnlyNormalLetters(input));
    }

    @Test
    public void containsOnlyNormalLettersFailesComma() {
        String input = "foo,bar,baz";
        assertFalse(Text.containsOnlyNormalLetters(input));
    }

    @Test
    public void tabToSpace() {
        String input = "foo\tbar\tbaz";
        String actual = Text.tabToSpace(input);
        String expected = "foo    bar    baz";
        assertEquals(expected, actual);
    }

    @Test
    public void tabToSpaceWithoutTab() {
        String input = "foo bar baz";
        String actual = Text.tabToSpace(input);
        String expected = "foo bar baz";
        assertEquals(expected, actual);
    }

    @Test
    public void tabToSpaceEmptyInput() {
        String input = "";
        String actual = Text.tabToSpace(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void extractQuotetBraceParameters() {
        String input = "MM_openBrWindow('impressum.html','PopUp','scrollbars=yes,width=560,height=565')";

        List<String> list = Text.extractQuotetBraceParameters(input);
        assertEquals(3, list.size());
        assertEquals("impressum.html", list.get(0));
        assertEquals("PopUp", list.get(1));
        assertEquals("scrollbars=yes,width=560,height=565", list.get(2));
    }

    @Test
    public void extractQuotetBraceParametersWithSpace() {
        String input = "  MM_openBrWindow('impressum.html','PopUp','scrollbars=yes,width=560,height=565')  ";

        List<String> list = Text.extractQuotetBraceParameters(input);
        assertEquals(3, list.size());
        assertEquals("impressum.html", list.get(0));
        assertEquals("PopUp", list.get(1));
        assertEquals("scrollbars=yes,width=560,height=565", list.get(2));
    }

    @Test
    public void extractQuotetBraceParametersWithMoreSpace() {
        String input = "  MM_openBrWindow   ('impressum.html','PopUp','scrollbars=yes,width=560,height=565')  ";

        List<String> list = Text.extractQuotetBraceParameters(input);
        assertEquals(3, list.size());
        assertEquals("impressum.html", list.get(0));
        assertEquals("PopUp", list.get(1));
        assertEquals("scrollbars=yes,width=560,height=565", list.get(2));
    }

    @Test (expected = IllegalArgumentException.class)
    public void extractQuotetBraceParametersWithTwoOpeningBraces() {
        String input = "MM_openBrWindow('impressum.html,('PopUp','scrollbars=yes,width=560,height=565')";

        Text.extractQuotetBraceParameters(input);
    }

    @Test (expected = IllegalArgumentException.class)
    public void extractQuotetBraceParametersWithNoOpeningBraces() {
        String input = "MM_openBrWindow['impressum.html,'PopUp','scrollbars=yes,width=560,height=565')";

        Text.extractQuotetBraceParameters(input);
    }

    @Test (expected = IllegalArgumentException.class)
    public void extractQuotetBraceParametersWithTwoClosingBraces() {
        String input = "MM_openBrWindow('impressum.html),'PopUp','scrollbars=yes,width=560,height=565')";

        Text.extractQuotetBraceParameters(input);
    }

    @Test (expected = IllegalArgumentException.class)
    public void extractQuotetBraceParametersWithNoClosingBraces() {
        String input = "MM_openBrWindow('impressum.html,'PopUp','scrollbars=yes,width=560,height=565']";

        Text.extractQuotetBraceParameters(input);
    }

    @Test
    public void removeTextAtFrontIfStartsWith() {
        String frontToRemove = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum\\data\\";
        String textToChange = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum\\data\\"
                + "splitted_batches\\20170506_mtest\\20170506_mtest_part25";
        String actual = Text.removeTextAtFrontIfStartsWith(textToChange, frontToRemove);
        String expected = "splitted_batches\\20170506_mtest\\20170506_mtest_part25";
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtFrontIfStartsWithRemoveEmptyString() {
        String frontToRemove = "";
        String textToChange = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum\\data\\"
                + "splitted_batches\\20170506_mtest\\20170506_mtest_part25";
        String actual = Text.removeTextAtFrontIfStartsWith(textToChange, frontToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtFrontIfStartsWithRemoveWrongStart() {
        String frontToRemove = "\\\\Cluster1\\Projekte1\\";
        String textToChange = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum\\data\\"
                + "splitted_batches\\20170506_mtest\\20170506_mtest_part25";
        String actual = Text.removeTextAtFrontIfStartsWith(textToChange, frontToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtFrontIfStartsWithRemoveFromEmptyString() {
        String frontToRemove = "\\\\Cluster1\\Projekte1\\";
        String textToChange = "";
        String actual = Text.removeTextAtFrontIfStartsWith(textToChange, frontToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtFrontIfStartsWithRemoveEmptyFromEmptyString() {
        String frontToRemove = "";
        String textToChange = "";
        String actual = Text.removeTextAtFrontIfStartsWith(textToChange, frontToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtEndIfEndsWith() {
        String endToRemove = "bar.baz";
        String textToChange = "bla.foo.bar.baz";
        String actual = Text.removeTextAtEndIfEndsWith(textToChange, endToRemove);
        String expected = "bla.foo.";
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtEndIfEndsWithRemoveEmptyString() {
        String endToRemove = "";
        String textToChange = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum\\data\\"
                + "splitted_batches\\20170506_mtest\\20170506_mtest_part25";
        String actual = Text.removeTextAtEndIfEndsWith(textToChange, endToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtEndIfEndsWithRemoveWrongStart() {
        String endToRemove = "bar.baz1789";
        String textToChange = "bla.foo.bar.baz";
        String actual = Text.removeTextAtEndIfEndsWith(textToChange, endToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtEndIfEndsWithRemoveFromEmptyString() {
        String endToRemove = "bar.baz";
        String textToChange = "";
        String actual = Text.removeTextAtEndIfEndsWith(textToChange, endToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeTextAtEndIfEndsWithRemoveEmptyFromEmptyString() {
        String endToRemove = "";
        String textToChange = "";
        String actual = Text.removeTextAtEndIfEndsWith(textToChange, endToRemove);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromStringWithLineBreak() {
        String textToChange = "abc" + Text.LINE_BREAK;
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "abc";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromStringWithN() {
        String textToChange = "abc" + "\n";
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "abc";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromStringWithMultipleLineBreaks() {
        String textToChange = "abc" + Text.LINE_BREAK
                + "def" + Text.LINE_BREAK
                + "ghi" + Text.LINE_BREAK;
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "abc" + Text.LINE_BREAK
                + "def" + Text.LINE_BREAK
                + "ghi";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromStringWithMultipleNs() {
        String textToChange = "abc" + "\n"
                + "def" + "\n"
                + "ghi" + "\n";
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "abc" + "\n"
                + "def" + "\n"
                + "ghi";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromStringWithoutLineBreaks() {
        String textToChange = "abc";
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromEmptyString() {
        String textToChange = "";
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = textToChange;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithFromEmptyStringWithLineBreak() {
        String textToChange = "" + Text.LINE_BREAK;
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithRemoveEmptyFromStringWithN() {
        String textToChange = "" + "\n";
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithRemoveEmptyFromStringWithLineBreakAndN() {
        String textToChange = "" + Text.LINE_BREAK + "\n";
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "" + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLineBreakAtEndIfEndsWithRemoveEmptyFromStringWithNAndLineBreak() {
        String textToChange = ""  + "\n"+ Text.LINE_BREAK;
        String actual = Text.removeLineBreakAtEndIfEndsWith(textToChange);
        String expected = "" + "\n";
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchText() {
        String text = "foo bar baz";
        String searchText = "bar";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = "bar baz";
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchTextSearchTextEmpty() {
        String text = "foo bar baz";
        String searchText = "";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchTextTextEmpty() {
        String text = "";
        String searchText = "bar";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchTextBothEmpty() {
        String text = "";
        String searchText = "";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchTextNotContainedSearchText() {
        String text = "foo bar baz";
        String searchText = "huhu";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchTextMultipleContainedSearchText() {
        String text = "foo bar bar baz";
        String searchText = "bar";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = "bar bar baz";
        assertEquals(expected, actual);
    }

    @Test
    public void removePartBeforeSearchTextSearchTextAtStart() {
        String text = "foo bar baz";
        String searchText = "foo";
        String actual = Text.removePartBeforeSearchText(text, searchText);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void unifyLineBreaksEmptyInput() {
        String text = "";
        String actual = Text.unifyLineBreaks(text);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void unifyLineBreaksWithoutBreaks() {
        String text = "Dies ist ein schöner Text ohne Umbrüche!";
        String actual = Text.unifyLineBreaks(text);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void unifyLineBreaksWithLineBreakBreaks() {
        String text = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        String actual = Text.unifyLineBreaks(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void unifyLineBreaksWithNBreaks() {
        String text = "Dies ist ein\nText mit\nUmbrüchen!";
        String actual = Text.unifyLineBreaks(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void unifyLineBreaksWithRNBreaks() {
        String text = "Dies ist ein\r\nText mit\r\nUmbrüchen!";
        String actual = Text.unifyLineBreaks(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void unifyLineBreaksWithMultipleLineBreakBreaks() {
        String text = "Dies ist ein" + LINE_BREAK + LINE_BREAK + "Text mit" + LINE_BREAK
                + LINE_BREAK + LINE_BREAK + LINE_BREAK + LINE_BREAK + "Umbrüchen!";
        String actual = Text.unifyLineBreaks(text);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesEmptyInput() {
        String text = "";
        String actual = Text.removeEmptyLines(text);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithoutBreaks() {
        String text = "Dies ist ein schöner Text ohne Umbrüche!";
        String actual = Text.removeEmptyLines(text);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithLineBreakBreaks() {
        String text = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithNBreaks() {
        String text = "Dies ist ein\nText mit\nUmbrüchen!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithRNBreaks() {
        String text = "Dies ist ein\r\nText mit\r\nUmbrüchen!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithRNBreaks2() {
        String text = "Dies ist ein\r\nText mit\r\nUmbrüchen!";
        String actual = Text.removeEmptyLines(text);
        String expected = text; // "\r\n" == LINE_BREAK !
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithMultipleBreaks() {
        String text = "Dies ist ein" + LINE_BREAK + LINE_BREAK + "Text mit" + LINE_BREAK
                + LINE_BREAK + LINE_BREAK + LINE_BREAK + LINE_BREAK + "Umbrüchen!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesWithMultipleBreaksWithSpaces() {
        String text = "Dies ist ein" + LINE_BREAK + "   " + LINE_BREAK + "Text mit" + LINE_BREAK
                + LINE_BREAK + "     " + LINE_BREAK + LINE_BREAK + " " + LINE_BREAK + "Umbrüchen!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein" + LINE_BREAK + "Text mit" + LINE_BREAK + "Umbrüchen!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFront() {
        String text = LINE_BREAK + "Dies ist ein Text mit Umbruch vorn!!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein Text mit Umbruch vorn!!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontWithSpace() {
        String text = "    " + LINE_BREAK + "Dies ist ein Text mit Umbruch vorn!!";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein Text mit Umbruch vorn!!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtRear() {
        String text = "Dies ist ein Text mit Umbruch hinten!!" + LINE_BREAK;
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein Text mit Umbruch hinten!!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtRearWithSpace() {
        String text = "Dies ist ein Text mit Umbruch hinten!!" + LINE_BREAK + "    ";
        String actual = Text.removeEmptyLines(text);
        String expected = "Dies ist ein Text mit Umbruch hinten!!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndRemoveFrontAndRearWithNormalLineBreak() {
        String text = "   " + LINE_BREAK + "   \t" + LINE_BREAK + "    abc    " + LINE_BREAK
                + "   " + LINE_BREAK;
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "    abc    ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndRemoveFrontAndRearWithNormalLineBreakEmptyLinesInside() {
        String text = "   " + LINE_BREAK + "   \t" + LINE_BREAK + "    abc    " + LINE_BREAK
                + "   " + LINE_BREAK + "    def    " + LINE_BREAK
                + "   " + LINE_BREAK;
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "    abc    " + LINE_BREAK
                + "   " + LINE_BREAK + "    def    ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndRemoveFrontAndRearWithBackslashN() {
        String text = "   " + "\n" + "   \t" + "\n" + "    abc    " + "\n"
                + "   " + "\n";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "    abc    ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndRemoveFrontAndRearWithBackslashRN() {
        String text = "   " + "\r\n" + "   \t" + "\r\n" + "    abc    " + "\r\n"
                + "   " + "\r\n";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "    abc    ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndRemoveFrontAndRearWithMixedLineBreakEmptyLinesInside() {
        String text = "   " + LINE_BREAK + "   \t" + "\n" + "    abc    " + "\r\n"
                + "   " + LINE_BREAK + "    def    " + "\n"
                + "   " + "\r\n";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "    abc    " + LINE_BREAK
                + "   " + LINE_BREAK + "    def    ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndNoRemoveWithNormalLineBreakEmptyLinesInside() {
        String text = "    abc    " + LINE_BREAK
                + "   " + LINE_BREAK + "    def    ";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = text;
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndNoRemoveWithBackslashNEmptyLinesInside() {
        String text = "    abc    " + "\n"
                + "   " + "\n" + "    def    ";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "    abc    " + LINE_BREAK
                + "   " + LINE_BREAK + "    def    ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndEmptyString() {
        String text = "";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyLinesAtFrontAndEndWhitespaces() {
        String text = "     \t    \t    ";
        String actual = Text.removeEmptyLinesAtFrontAndEnd(text);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeShortLinesAtFront() {
        String text = "abc" + LINE_BREAK + "Dies ist ein Text mit Umbruch vorn!!";
        String actual = Text.removeShortLines(text, 3);
        String expected = "Dies ist ein Text mit Umbruch vorn!!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeShortLinesAtMiddle() {
        String text = "Dies ist ein Text mit Umbruch vorn und hinten!"
                + LINE_BREAK + "abc"
                + LINE_BREAK + "Dies ist ein Text mit Umbruch vorn und hinten!";
        String actual = Text.removeShortLines(text, 3);
        String expected = "Dies ist ein Text mit Umbruch vorn und hinten!"
                + LINE_BREAK + "Dies ist ein Text mit Umbruch vorn und hinten!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeTwoShortLinesAtMiddle() {
        String text = "Dies ist ein Text mit Umbruch vorn und hinten!"
                + LINE_BREAK + "abc"
                + LINE_BREAK + "abc"
                + LINE_BREAK + "Dies ist ein Text mit Umbruch vorn und hinten!";
        String actual = Text.removeShortLines(text, 3);
        String expected = "Dies ist ein Text mit Umbruch vorn und hinten!"
                + LINE_BREAK + "Dies ist ein Text mit Umbruch vorn und hinten!";
        assertEquals(expected, actual);
    }

    @Test
    public void removeShortLinesAtRear() {
        String text = "Dies ist ein Text mit Umbruch hinten!" + LINE_BREAK + "abc";
        String actual = Text.removeShortLines(text, 3);
        String expected = "Dies ist ein Text mit Umbruch hinten!";
        assertEquals(expected, actual);
    }

    @Test
    public void eachCharFromStringEmptyInput() {
        String text = "";
        List<String> characters = Text.eachCharFromString(text);
        assertEquals(0, characters.size());
    }

    @Test
    public void eachCharFromStringSingleCharacter() {
        String text = "A";
        List<String> characters = Text.eachCharFromString(text);
        assertEquals(1, characters.size());
        assertEquals("A", characters.get(0));
    }

    @Test
    public void eachCharFromStringAB() {
        String text = "AB";
        List<String> characters = Text.eachCharFromString(text);
        assertEquals(2, characters.size());
        assertEquals("A", characters.get(0));
        assertEquals("B", characters.get(1));
    }

    @Test
    public void eachCharFromStringABC() {
        String text = "ABC";
        List<String> characters = Text.eachCharFromString(text);
        assertEquals(3, characters.size());
        assertEquals("A", characters.get(0));
        assertEquals("B", characters.get(1));
        assertEquals("C", characters.get(2));
    }

    @Test
    public void eachCharFromStringAToZ() {
        String text = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        assertEquals(26, text.length());
        List<String> characters = Text.eachCharFromString(text);
        assertEquals(text.length(), characters.size());
        assertEquals("A", characters.get(0));
        assertEquals("B", characters.get(1));
        assertEquals("C", characters.get(2));
        assertEquals("D", characters.get(3));
        assertEquals("E", characters.get(4));
        assertEquals("F", characters.get(5));
        assertEquals("G", characters.get(6));
        assertEquals("H", characters.get(7));
        assertEquals("I", characters.get(8));
        assertEquals("J", characters.get(9));
        assertEquals("K", characters.get(10));
        assertEquals("L", characters.get(11));
        assertEquals("M", characters.get(12));
        assertEquals("N", characters.get(13));
        assertEquals("O", characters.get(14));
        assertEquals("P", characters.get(15));
        assertEquals("Q", characters.get(16));
        assertEquals("R", characters.get(17));
        assertEquals("S", characters.get(18));
        assertEquals("T", characters.get(19));
        assertEquals("U", characters.get(20));
        assertEquals("V", characters.get(21));
        assertEquals("W", characters.get(22));
        assertEquals("X", characters.get(23));
        assertEquals("Y", characters.get(24));
        assertEquals("Z", characters.get(25));
    }

    @Test
    public void findFirstPosition() {
        List<String> starts = CollectionsHelper.buildListFrom("a", "A");
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        FoundSearch found = Text.findFirstPosition(text, starts);
        int actual = found.getIndex();
        int expected = 8;
        assertEquals(expected, actual);
        assertEquals("A", found.getSearch());
    }

    @Test
    public void findFirstPositionNotFound() {
        List<String> starts = CollectionsHelper.buildListFrom("z", "Z");
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        FoundSearch found = Text.findFirstPosition(text, starts);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionNotFoundWithEmptyList() {
        List<String> starts = CollectionsHelper.buildListFrom();
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        FoundSearch found = Text.findFirstPosition(text, starts);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionNotFoundInEmptyString() {
        List<String> starts = CollectionsHelper.buildListFrom("z", "Z");
        String text = "";
        FoundSearch found = Text.findFirstPosition(text, starts);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionNotFoundInEmptyStringAndEmptyString() {
        List<String> starts = CollectionsHelper.buildListFrom();
        String text = "";
        FoundSearch found = Text.findFirstPosition(text, starts);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionIndex() {
        List<String> starts = CollectionsHelper.buildListFrom("a", "A");
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        int fromIndex = 9;
        FoundSearch found = Text.findFirstPosition(text, starts, fromIndex);
        int actual = found.getIndex();
        int expected = 15;
        assertEquals(expected, actual);
        assertEquals("a", found.getSearch());
    }

    @Test
    public void findFirstPositionIndexNotFound() {
        List<String> starts = CollectionsHelper.buildListFrom("z", "Z");
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        int fromIndex = 9;
        FoundSearch found = Text.findFirstPosition(text, starts, fromIndex);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionIndexNotFoundBecauseOfHighIndex() {
        List<String> starts = CollectionsHelper.buildListFrom("a", "A");
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        int fromIndex = 100;
        FoundSearch found = Text.findFirstPosition(text, starts, fromIndex);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionIndexNotFoundWithEmptyList() {
        List<String> starts = CollectionsHelper.buildListFrom();
        String text = "BbbdghshAshdhsjajjsdjjajjsdkadjAsdjjaa";
        int fromIndex = 5;
        FoundSearch found = Text.findFirstPosition(text, starts, fromIndex);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionIndexNotFoundInEmptyString() {
        List<String> starts = CollectionsHelper.buildListFrom("z", "Z");
        String text = "";
        int fromIndex = 5;
        FoundSearch found = Text.findFirstPosition(text, starts, fromIndex);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void findFirstPositionIndexNotFoundInEmptyStringAndEmptyString() {
        List<String> starts = CollectionsHelper.buildListFrom();
        String text = "";
        int fromIndex = 0;
        FoundSearch found = Text.findFirstPosition(text, starts, fromIndex);
        int actual = found.getIndex();
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void startsWithIgnoreCase() {
        String text = "Jahresbilanz 2018";
        String start = "JAHRESBIlanz";
        assertTrue(Text.startsWithIgnoreCase(text, start));
    }

    @Test
    public void startsWithIgnoreCaseUmlaut() {
        String text = "ÄäÜüÖöß Haha!";
        String start = "ääÜÜööß";
        assertTrue(Text.startsWithIgnoreCase(text, start));
    }

    @Test
    public void startsWithIgnoreCaseFailing() {
        String text = "foo bar baz";
        String start = "bar baz";
        assertFalse(Text.startsWithIgnoreCase(text, start));
    }

    @Test
    public void removeSlashStarComments() {
        String text = "Ein Text /* mit einem schönen */Kommentar";
        String actual = Text.removeSlashStarComments(text );
        String expected = "Ein Text Kommentar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSlashStarCommentsAtStart() {
        String text = "/* mit einem schönen */Kommentar";
        String actual = Text.removeSlashStarComments(text );
        String expected = "Kommentar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSlashStarCommentsAtEnd() {
        String text = "Ein Text /* mit einem schönen */";
        String actual = Text.removeSlashStarComments(text );
        String expected = "Ein Text ";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSlashStarCommentsMultiple() {
        String text = "/* bla /* sfawawd /**/Ein Text /* mit einem schönen */Kommentar/*     awdaw */";
        String actual = Text.removeSlashStarComments(text );
        String expected = "Ein Text Kommentar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpacesFormEmptyString() {
        String input = "";
        String actual = Text.removeSpaces(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpacesFormStringWithOnlySpaces() {
        String input = "               ";
        String actual = Text.removeSpaces(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpacesFormStringWithoutSpaces() {
        String input = "foo";
        String actual = Text.removeSpaces(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpacesFormStringWithSpacesInFront() {
        String input = "   foo";
        String actual = Text.removeSpaces(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpacesFormStringWithSpacesBehind() {
        String input = "foo   ";
        String actual = Text.removeSpaces(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpacesFormStringWithSpacesInFrontAndBehind() {
        String input = "            foo   ";
        String actual = Text.removeSpaces(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void removeSpaces() {
        String input = "            foo   bar     baz     ";
        String actual = Text.removeSpaces(input);
        String expected = "foobarbaz";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanUselessWhitspaceEmptyInput() {
        String input = "";
        String actual = Text.cleanUselessWhitspace(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanUselessWhitspaceInputWithoutSpaces() {
        String input = "foobarbaz";
        String actual = Text.cleanUselessWhitspace(input);
        String expected = "foobarbaz";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanUselessWhitspace() {
        String input = "            foo   bar     baz     ";
        String actual = Text.cleanUselessWhitspace(input);
        String expected = "foo bar baz";
        assertEquals(expected, actual);
    }

    @Test
    public void isStartOfWordBeginOfText() {
        String text = "foo bar baz";
        int index = 0;
        assertTrue(Text.isStartOfWord(index, text));
    }

    @Test
    public void isStartOfWordBeforeBeginOfText() {
        String text = "foo bar baz";
        int index = -3;
        assertFalse(Text.isStartOfWord(index, text));
    }

    @Test
    public void isStartOfWordAtEndOfText() {
        String text = "foo bar baz";
        int index = text.length();
        assertFalse(Text.isStartOfWord(index, text));
    }

    @Test
    public void isStartOfWordBeginOfWord() {
        String text = "foo bar baz";
        int index = 4;
        assertTrue(Text.isStartOfWord(index, text));
    }

    @Test
    public void isStartOfWordMiddleOfWord() {
        String text = "foo bar baz";
        int index = 5;
        assertFalse(Text.isStartOfWord(index, text));
    }

    @Test
    public void isStartOfWordBeginOfWordAfterSpecialChar() {
        String text = "foo 'bar' baz";
        int index = 5;
        assertTrue(Text.isStartOfWord(index, text));
    }

    @Test
    public void isEndOfWordEndOfText() {
        String text = "foo bar baz";
        int index = text.length();
        assertTrue(Text.isEndOfWord(index, text));
    }

    @Test
    public void isEndOfWordEndOfWord() {
        String text = "foo bar baz";
        int index = 3;
        assertTrue(Text.isEndOfWord(index, text));
    }

    @Test
    public void isEndOfWordMiddleOfWord() {
        String text = "foo bar baz";
        int index = 2;
        assertFalse(Text.isEndOfWord(index, text));
    }

    @Test
    public void isEndOfWordEndOfWordBeforeSpecialChar() {
        String text = "foo 'bar' baz";
        int index = 9;
        assertTrue(Text.isEndOfWord(index, text));
    }

    @Test
    public void isEndOfWordStartOfText() {
        String text = "foo bar baz";
        int index = 0;
        assertFalse(Text.isEndOfWord(index, text));
    }

    @Test
    public void isEndOfWordBeforeStartOfText() {
        String text = "foo bar baz";
        int index = -13;
        assertFalse(Text.isEndOfWord(index, text));
    }

    @Test
    public void isEndOfWordAfterEndOfText() {
        String text = "foo bar baz";
        int index = 66;
        assertFalse(Text.isEndOfWord(index, text));
    }

    @Test
    public void isStartAndEndOfWordOfOneWord() {
        String text = "foo bar baz";
        int startIndex = 4;
        int endIndex = 7;
        assertTrue(Text.isStartAndEndOfWord(startIndex, endIndex, text));
    }

    @Test
    public void isStartAndEndOfWordOfTwoWord() {
        String text = "foo bar baz";
        int startIndex = 4;
        int endIndex = 11;
        assertTrue(Text.isStartAndEndOfWord(startIndex, endIndex, text));
    }

    @Test
    public void isNotStartAndEndOfWordOfOneWord() {
        String text = "foo bar baz";
        int startIndex = 4;
        int endIndex = 8;
        assertFalse(Text.isStartAndEndOfWord(startIndex, endIndex, text));
    }

    @Test
    public void escapeDoubleQuotes() {
        String input = "Text \"mit\" Anführungszeichen.";
        String actual = Text.escapeDoubleQuotes(input);
        String expected = "Text \\\"mit\\\" Anführungszeichen.";
        assertEquals(expected, actual);
    }

    @Test
    public void isUpperCase() {
        String text = "ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜÁÉÍÓÚÀÈÌÒÙÂÊÎÔÛ";
        assertTrue(Text.isUpperCase(text));
    }

    @Test
    public void isUpperCase2() {
        String text = "ABCDEF, - 'GHIJKLMNOPQRSTUVWX' / YZÄÖÜÁÉÍÓÚÀ ÈÌÒÙÂÊÎÔÛ 2134*@$&%/!\"?´`^";
        assertTrue(Text.isUpperCase(text));
    }

    @Test
    public void isNoUpperCase() {
        String text = "abcdefghijklmnopqrstuvwxyzäöüßáéíóúàèìòùâêîôû";
        assertFalse(Text.isUpperCase(text));
    }

    @Test
    public void isNoUpperCase2() {
        String text = "ò";
        assertFalse(Text.isUpperCase(text));
    }

    @Test
    public void isCamelCaseEmptyString() {
        String text = "";
        assertFalse(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseNormalText() {
        String text = "Foo";
        assertFalse(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseNormalTextWithBlanks() {
        String text = "Foo Bar";
        assertFalse(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseTwoCamels() {
        String text = "FooBar";
        assertTrue(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseThreeCamels() {
        String text = "FooBarBaz";
        assertTrue(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseFourCamels() {
        String text = "FooBarBazFour";
        assertTrue(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseFourCamelsWithOnlyOneSmallLetterBetween() {
        String text = "FoBaBaFo";
        assertTrue(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseFourCamelsWithoutOnlyOneSmallLetterBetween() {
        String text = "FooBarBazCFour";
        assertFalse(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseFullBig() {
        String text = "ABIGTEXT";
        assertFalse(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseFullSmall() {
        String text = "asmalltext";
        assertFalse(Text.isCamelCase(text));
    }

    @Test
    public void isCamelCaseWithDotsWithDotAtEnd() {
        String text = "Grund.Wert.Vermögen.";
        assertTrue(Text.isCamelCaseWithDots(text));
    }

    @Test
    public void isCamelCaseWithDotWithoutDotAtEnd() {
        String text = "Grund.Wert.Vermögen";
        assertTrue(Text.isCamelCaseWithDots(text));
    }

    @Test
    public void isCamelCaseWithDotWithCamelCase() {
        String text = "GrundWertVermögen";
        assertFalse(Text.isCamelCaseWithDots(text));
    }

    @Test
    public void isCamelCaseWithDotEmpty() {
        String text = "";
        assertFalse(Text.isCamelCaseWithDots(text));
    }

    @Test
    public void isCamelCaseWithDotFullBig() {
        String text = "ABCDEFGHI";
        assertFalse(Text.isCamelCaseWithDots(text));
    }

    @Test
    public void isCamelCaseWithDotFullSmall() {
        String text = "abcdefhgi";
        assertFalse(Text.isCamelCaseWithDots(text));
    }

    @Test
    public void findAllPositionsOfCompleteWordsWordNotContained() {
        String searchText = "foo";
        String text = "foofoofoo";
        List<Integer> list = Text.findAllPositionsOfCompleteWords(searchText, text);
        assertEquals(0, list.size());
    }

    @Test
    public void findAllPositionsOfCompleteWords() {
        String searchText = "foo";
        String text = "foo foo foo";
        List<Integer> list = Text.findAllPositionsOfCompleteWords(searchText, text);
        assertEquals(3, list.size());
        assertEquals(Integer.valueOf(0), list.get(0));
        assertEquals(Integer.valueOf(4), list.get(1));
        assertEquals(Integer.valueOf(8), list.get(2));
    }

    @Test
    public void findAllPositionsOfCompleteWords2() {
        String searchText = "foo";
        String text = "foo fooo foo";
        List<Integer> list = Text.findAllPositionsOfCompleteWords(searchText, text);
        assertEquals(2, list.size());
        assertEquals(Integer.valueOf(0), list.get(0));
        assertEquals(Integer.valueOf(9), list.get(1));
    }

    @Test
    public void firstLoweredChar() {
        String input = "Übel";
        String actual = Text.firstLoweredChar(input);
        String expected = "ü";
        assertEquals(expected, actual);
    }

    @Test
    public void firstLoweredCharAlreadyLowered() {
        String input = "äpfel";
        String actual = Text.firstLoweredChar(input);
        String expected = "ä";
        assertEquals(expected, actual);
    }

    @Test
    public void firstLoweredCharEmpty() {
        String input = "   Äpfel   ";
        String actual = Text.firstLoweredChar(input);
        String expected = " ";
        assertEquals(expected, actual);
    }

    @Test
    public void firstLoweredCharNumber() {
        String input = "1234567";
        String actual = Text.firstLoweredChar(input);
        String expected = "1";
        assertEquals(expected, actual);
    }

    @Test
    public void wordToLettersEmptyInput() {
        String input = "";
        List<String> letters = Text.wordToLetters(input);
        assertEquals(0, letters.size());
    }

    @Test
    public void wordToLetters() {
        String input = "abc";
        List<String> letters = Text.wordToLetters(input);
        assertEquals(3, letters.size());
        assertEquals("a", letters.get(0));
        assertEquals("b", letters.get(1));
        assertEquals("c", letters.get(2));
    }

    @Test
    public void isQuoted() {
        assertFalse(Text.isQuoted(""));
        assertFalse(Text.isQuoted("foo"));
        assertFalse(Text.isQuoted("\"foo"));
        assertFalse(Text.isQuoted("foo\""));
        assertTrue(Text.isQuoted("\"foo\""));
        assertTrue(Text.isQuoted("\"\""));
    }

    @Test
    public void deQuoteQuotedString() {
        String input = "\"foo\"";
        String actual = Text.deQuoteQuotedString(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test (expected = IllegalArgumentException.class)
    public void deQuoteQuotedStringOnNotQuotedString() {
        Text.deQuoteQuotedString("foo");
    }

}
