package de.duehl.math.graph.ged.animation.io;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.text.NumberString;
import de.duehl.basics.text.Text;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.io.ged4.Ged4Helper;
import de.duehl.math.graph.ged.graph.io.ged4.Variable;

import static de.duehl.math.graph.ged.animation.io.AnimationSaver.*;

public class AnimationLoader {

    private String filename;

    public List<Graph> load(String filename) {
        this.filename = filename;

        String content = FileHelper.readFileToString(filename, CHARSET);
        List<String> contentParts = splitAtSeparators(content);

        int numberOfGraphs = determineNumberOfGraphsFromFirstPart(contentParts.get(0));
        if (numberOfGraphs + 1 != contentParts.size()) {
            throw new RuntimeException("Anzahl der graphen laut Kopfteil des Animationsteils, "
                    + numberOfGraphs + ", weicht von der Anzahl der gespeicherten Graphen, "
                    + (contentParts.size() - 1) + " ab!");
        }

        List<Graph> graphs = new ArrayList<>();
        for (int index = 1; index < contentParts.size(); ++index) {
            Graph graph = new Graph();
            graph.loadFromGed4Format(contentParts.get(index));
            graphs.add(graph);
        }

        return graphs;
    }

    private List<String> splitAtSeparators(String content) {
        String regex = "\\s*" + Pattern.quote(SEPARATOR) + "\\s*";
        List<String> contentParts = Text.splitBy(content, regex);
        if (contentParts.isEmpty()) {
            throw new RuntimeException(
                    "Keine am Trenner aufgeteilte Teile in " + filename + " gefunden!");
        }

        return contentParts;
    }

    private int determineNumberOfGraphsFromFirstPart(String firstPart) {
        List<String> lines = Text.splitByLineBreaks(firstPart);
        for (int index = 0; index < lines.size(); ++index) {
            String line = lines.get(index);

            Variable variable = Ged4Helper.determineVariable(line, line, index);
            String key = variable.getName();
            String value = variable.getValue();
            switch (key) {
                case NUMBER_OF_GRAPHS_IN_ANIMATION :
                    return NumberString.parseInt(value, "Kann die Anzahl der Graphen nicht parsen!");
            }
        }
        throw new RuntimeException("Keine  Anzhal der Graphen gespeichert.");
    }

}
