package de.duehl.math.graph.ged.graph.io.ged4;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.text.Text;

/**
 * Diese Klasse stellt eine Variable mit Bezeichnung und Wert dar.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class Variable {

    /** Name oder Bezeichnung der Variablen. */
    private final String name;

    /** Wert der Variablen. */
    private final String value;

    /**
     * Konstruktor.
     *
     * @param name
     *            Name oder Bezeichnung der Variablen.
     * @param value
     *            Wert der Variablen.
     */
    public Variable(String name, String value) {
        this.name = name;
        this.value = value;
    }

    /**
     * Konstruktor.
     *
     * @param name
     *            Name oder Bezeichnung der Variablen.
     * @param value
     *            Wert der Variablen.
     */
    public Variable(String name, int value) {
        this.name = name;
        this.value = Integer.toString(value);
    }

    /**
     * Konstruktor.
     *
     * @param name
     *            Name oder Bezeichnung der Variablen.
     * @param value
     *            Wert der Variablen.
     */
    public Variable(String name, double value) {
        this.name = name;
        this.value = Double.toString(value);
    }

    public String getName() {
        return name;
    }

    public String getValue() {
        return value;
    }

    @Override
    public String toString() {
        return name + "=" + value;
    }

    /**
     * Trennt Name und Wert der Variablen in der Form "name=wert" am Gleichheitszeichen.
     *
     * @param nameAndValueEquation
     *            Name und Wert der Variablen in der Form "name=wert".
     */
    public static Variable splitVariableByEqualsSign(String nameAndValueEquation) {
        return splitVariableBy(nameAndValueEquation, " *= *");
    }

    /**
     * Trennt Name und Wert der Variablen in der Form "name:wert" am Doppelpunkt.
     *
     * @param nameAndValueEquation
     *            Name und Wert der Variablen in der Form "name:wert".
     */
    public static Variable splitVariableByColon(String nameAndValueEquation) {
        return splitVariableBy(nameAndValueEquation, " *: *");
    }

    private static Variable splitVariableBy(String nameAndValueEquation, String regex) {
        List<String> parts = Text.splitBy(nameAndValueEquation, regex);
        if (parts.size() != 2) {
            throw new RuntimeException("Wert der Gleichung lässt sich nicht an '" + regex
                     + "' trennen!");
        }
        else {
            String name = parts.get(0);
            String value = parts.get(1);
            return new Variable(name, value);
        }
    }

    /** Erstellt aus einer Liste von Variablen eine durch Semikolon getrennte Liste von Gleichungen. */
    public static String variableListToSemicolonJoinedEquationList(List<Variable> variables) {
        List<String> equations = variableListToEquationList(variables);
        return Text.joinWithSemicolon(equations);
    }

    /** Erstellt aus einer Liste von Variablen eine Liste von Gleichungen. */
    public static List<String> variableListToEquationList(List<Variable> variables) {
        List<String> equations = new ArrayList<>();
        for (Variable variable : variables) {
            equations.add(variable.toString());
        }
        return equations;
    }

}
