package de.duehl.math.graph.ged.ui.creation.menu;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.swing.ui.menu.MyMenuItem;
import de.duehl.swing.ui.menu.SingleMenuCreation;

/**
 * Diese Klasse stellt die Methoden zur Verfügung, um das Extra-Menü für die grafische Oberfläche
 * zu erstellen.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class ViewMenuCreation implements SingleMenuCreation {

    /** Verbindung zur Programmlogik. */
    private final Logic logic;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Programmlogik.
     */
    public ViewMenuCreation(Logic logic) {
        this.logic = logic;
    }

    /** Erzeugt das Ansichtmenü. */
    @Override
    public JMenu createTheMenu() {
        JMenu menu = new JMenu("Ansicht");
        menu.setMnemonic('a');

        menu.add(createSetCanwaxSize420x300MenuItem());
        menu.add(createSetCanwaxSize600x450MenuItem());
        menu.add(createSetCanwaxSize800x600MenuItem());
        menu.add(createSetCanwaxSize1024x768MenuItem());

        return menu;
    }

    private JMenuItem createSetCanwaxSize420x300MenuItem() {
        return new MyMenuItem("Canvasgröße auf 420x300 setzen")
                .mnemonic('3')
                //.accelerator(KeyEvent.VK_F5, 0)
                .actionListener(e -> logic.setViewSize(420, 300));
    }

    private JMenuItem createSetCanwaxSize600x450MenuItem() {
        return new MyMenuItem("Canvasgröße auf 600x450 setzen")
                .mnemonic('6')
                //.accelerator(KeyEvent.VK_F5, 0)
                .actionListener(e -> logic.setViewSize(600, 450));
    }

    private JMenuItem createSetCanwaxSize800x600MenuItem() {
        return new MyMenuItem("Canvasgröße auf 800x600 setzen")
                .mnemonic('8')
                //.accelerator(KeyEvent.VK_F5, 0)
                .actionListener(e -> logic.setViewSize(800, 600));
    }
    private JMenuItem createSetCanwaxSize1024x768MenuItem() {
        return new MyMenuItem("Canvasgröße auf 1024x768 setzen")
                .mnemonic('1')
                //.accelerator(KeyEvent.VK_F5, 0)
                .actionListener(e -> logic.setViewSize(1024, 768));
    }

}
