package de.duehl.math.graph.ged.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JPanel;

import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse öffnet einen kleinen Dialog zum Wählen einer Größe aus fest vorgegebenen Werten.
 *
 * @version 1.02     2019-05-24
 * @author Christian Dühl
 */

public class SizeChooser extends JDialog implements ActionListener {

    private static final long serialVersionUID = -1;

    private static final String VALUE_FIELD = "VALUE_FIELD";
    public static final int NO_VALUE = Integer.MIN_VALUE;

    private int size;

    private JFormattedTextField valueTextField;

    private int oldValue;

    /** Konstruktor */
    public SizeChooser() {
        this(-1);
    }

    /** Konstruktor */
    public SizeChooser(int oldValue) {
        size = NO_VALUE;
        this.oldValue = oldValue;
    }

    /**
     * Zeigt den Dialog an.
     *
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll
     *
     * @return Gewählte Größe oder -1, wenn der Dialog abgebrochen wurde.
     */
    public int showDialog(PixelPoint coords) {
        fillDialog();
        runDialog(coords);

        return size;
    }

    /** Baut die kleine Gui auf. */
    private void fillDialog() {
        if (oldValue > -1) {
            setTitle("Größe wählen (alter Wert ist " + oldValue + "):");
        }
        else {
            setTitle("Größe wählen:");
        }
        setModalityType(ModalityType.APPLICATION_MODAL);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setLayout(new BorderLayout());

        /* Zahlenfelder erzeugen: */
        JPanel upperPanel = new JPanel();
        add(upperPanel, BorderLayout.CENTER);

        upperPanel.setLayout(new GridLayout(5, 4));
        /*
         *     3   8  16  35
         *     4   9  18  40
         *     5  10  20  50
         *     6  12  25  75
         *     7  14  30 100
         */
        /*
        for (int i : new Integer[] { 3,   8,  16,  35,
                4,   9,  18,  40,
                5,  10,  20,  50,
                6,  12,  25,  75,
                7,  14,  30, 100}) {
        */
        for (int i : new Integer[] { 1,  6,  11,  16,  35,
                                     2,  7,  12,  18,  40,
                                     3,  8,  13,  20,  50,
                                     4,  9,  14,  25,  75,
                                     5, 10,  15,  30, 100}) {
            JButton button = new JButton(Integer.toString(i));
            button.setName("button" + Integer.toString(i));
            button.addActionListener(this);
            upperPanel.add(button);
        }


        JPanel lowerPanel = new JPanel();
        add(lowerPanel, BorderLayout.SOUTH);

        lowerPanel.setLayout(new BorderLayout());
        NumberFormat fmt = NumberFormat.getInstance();
        fmt.setMaximumFractionDigits(0); // no fractions - integer only
        fmt.setMinimumFractionDigits(0); // as above
        JFormattedTextField valueTextField = new JFormattedTextField(fmt);

        valueTextField.setBackground(Color.WHITE); // TODO Methoden aus GuiTools nehmen?
        //valueTextField.setForeground(foregroundColor);
        valueTextField.addActionListener(this);
        valueTextField.setName(VALUE_FIELD);
        lowerPanel.add(valueTextField, BorderLayout.CENTER);
        this.valueTextField = valueTextField;
    }

    /**
     * Zeigt den Dialog an.
     *
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll
     */
    private void runDialog(PixelPoint coords) {
        pack();
        int x = coords.getX();
        int y = coords.getY();
        setLocation(x, y);
        setVisible(true);
    }

    /** ActionHandler */
    @Override
    public void actionPerformed(ActionEvent event) {
        JComponent component = (JComponent) event.getSource();
        String name = component.getName();

        if (name.startsWith("button")) {
            String numberString = name.substring(6);
            size = Integer.parseInt(numberString);
        }
        else {
            switch(name) {
                case VALUE_FIELD:
                    String numberString = valueTextField.getText();
                    if (numberString.matches("\\d+")) {
                        size = Integer.parseInt(numberString);
                    }
                    else {
                        size = NO_VALUE;
                    }
                    break;
                default:
                    size = NO_VALUE;
                    System.err.println("Irgendwas in SizeChooser " +
                            "vergessen: name = '" + name + "'!");
                    break;
            }
        }

        setVisible(false);
    }

}
