package de.duehl.math.graph.ged.ui.elements;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JFrame;
import javax.swing.JPanel;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.ui.GedGuiTools;
import de.duehl.math.graph.ged.ui.elements.color.ColorSetter;
import de.duehl.math.graph.ged.ui.elements.color.DefaultOrOwnColorChooser;
import de.duehl.swing.ui.components.elements.HorizontalNumericTextFieldWithTitle;
import de.duehl.swing.ui.components.elements.TextFieldWithTitle;

/**
 * Diese Klasse stellt für den Ecken- und Kanten-Dialog Eingabefelder für einen String-Wert, X-
 * und Y-Abstand und eine Farbwahl mit Default-Schalter dar.
 *
 * @version 1.01     2019-05-27
 * @author Christian Dühl
 */

public class ValueDistanceAndColorChooser {

    private final JPanel panel;

    /** Text vor dem Eingabefeld für den Wert. */
    private final String titleOfValueInputField;

    /** Gibt an, ob zu Beginn die Default-Farbe verwendet wird. */
    private final boolean hasDefaultColor;

    /** Text vor dem Eingabefeld zum Abstand in X-Richtung. */
    private final String textForXDistance;

    /** Text vor dem Eingabefeld zum Abstand in Y-Richtung. */
    private final String textForYDistance;

    /** Text der auf der Checkbox zum an und Ausschalten des Default-Zustands angezeigt wird. */
    private final String textForToggleButton;

    /** Text der als Titel vor dem Farbbutton angezeigt wird. */
    private final String textForColorButtonTitle;

    /** Text der als Titel des Dialogs zur Farbauswahl angezeigt wird. */
    private final String textForColorChooserDialogTitle;

    /** Farbe des Objekts zum Zeitpunkt des Aufrufs. */
    private final Color actualSwingColor;

    /** Default-Farbe des Objekts. */
    private final GedColor defaultGedColor;

    /** Setzt die Farbe im Objekt. */
    private final ColorSetter colorSetter;

    /** Rahmen von GED zur relativen Positionierung des Farbauswahldialogs. */
    private final JFrame frame;

    private TextFieldWithTitle valueField;
    private TextFieldWithTitle xDistanceField;
    private TextFieldWithTitle yDistanceField;
    private DefaultOrOwnColorChooser chooser;

    /**
     * Konstruktor.
     *
     * @param titleOfValueInputField
     *            Text vor dem Eingabefeld für den Wert.
     * @param hasDefaultColor
     *            Gibt an, ob zu Beginn die Default-Farbe verwendet wird.
     * @param textForXDistance
     *            Text vor dem Eingabefeld zum Abstand in X-Richtung.
     * @param textForYDistance
     *            Text vor dem Eingabefeld zum Abstand in Y-Richtung.
     * @param textForToggleButton
     *            Text der auf der Checkbox zum an und Ausschalten des Default-Zustands angezeigt
     *            wird.
     * @param textForColorButtonTitle
     *            Text der als Titel vor dem Farbbutton angezeigt wird.
     * @param textForColorChooserDialogTitle
     *            Text der als Titel des Dialogs zur Farbauswahl angezeigt wird.
     * @param actualSwingColor
     *            Farbe des Objekts zum Zeitpunkt des Aufrufs.
     * @param defaultGedColor
     *            Default-Farbe des Objekts.
     * @param colorSetter
     *            Setzt die Farbe im Objekt.
     * @param frame
     *            Rahmen von GED zur relativen Positionierung des Farbauswahldialogs.
     */
    public ValueDistanceAndColorChooser(String titleOfValueInputField, boolean hasDefaultColor,
            String textForXDistance, String textForYDistance, String textForToggleButton,
            String textForColorButtonTitle, String textForColorChooserDialogTitle,
            Color actualSwingColor, GedColor defaultGedColor, ColorSetter colorSetter,
            JFrame frame) {
        this.titleOfValueInputField = titleOfValueInputField;
        this.hasDefaultColor = hasDefaultColor;
        this.textForXDistance = textForXDistance;
        this.textForYDistance = textForYDistance;
        this.textForToggleButton = textForToggleButton;
        this.textForColorButtonTitle = textForColorButtonTitle;
        this.textForColorChooserDialogTitle = textForColorChooserDialogTitle;
        this.actualSwingColor = actualSwingColor;
        this.defaultGedColor = defaultGedColor;
        this.colorSetter = colorSetter;
        this.frame = frame;


        panel = createValueDistanceAndColor();
        setEnabled(hasDefaultColor);
    }

    private JPanel createValueDistanceAndColor() {
        JPanel panel = new JPanel();
        panel.setLayout(GedGuiTools.createVerticalComponentLayout());

        panel.add(createValueFieldPart());
        panel.add(createXDistancePanel());
        panel.add(createYDistancePanel());
        panel.add(createColorPanel());

        return panel;
    }

    private Component createValueFieldPart() {
        valueField = createValueField(titleOfValueInputField);
        setStartValue("");
        return valueField.getPanel();
    }

    protected TextFieldWithTitle createValueField(String title) {
        return new TextFieldWithTitle(title);
    }

    public void setStartValue(String startValue) {
        valueField.setText(handleMinusInfinity(startValue));
    }

    private String handleMinusInfinity(String input) {
        if ("-Infinity".equals(input)) {
            return "0";
        }
        else {
            return input;
        }
    }

    private Component createXDistancePanel() {
        xDistanceField = new HorizontalNumericTextFieldWithTitle(textForXDistance);
        return xDistanceField.getPanel();
    }
    private Component createYDistancePanel() {
        yDistanceField = new HorizontalNumericTextFieldWithTitle(textForYDistance);
        return yDistanceField.getPanel();
    }

    public void setXDistance(int xDistance) {
        xDistanceField.setText(Integer.toString(xDistance));
    }

    public void setYDistance(int yDistance) {
        yDistanceField.setText(Integer.toString(yDistance));
    }

    private Component createColorPanel() {
        chooser = new DefaultOrOwnColorChooser(
                hasDefaultColor,
                textForToggleButton,
                textForColorButtonTitle,
                textForColorChooserDialogTitle,
                actualSwingColor,
                defaultGedColor,
                colorSetter,
                frame);

        return chooser.createPanel();
    }

    public JPanel getPanel() {
        return panel;
    }

    public void setEnabled(boolean enabled) {
        valueField.setEnabled(enabled);
        xDistanceField.setEnabled(enabled);
        yDistanceField.setEnabled(enabled);
        chooser.setEnabled(enabled);
    }

    public String getValue() {
        return valueField.getText();
    }

    public String getXDistanceFieldText() {
        return xDistanceField.getText();
    }

    public String getYDistanceFieldText() {
        return yDistanceField.getText();
    }

    public boolean isDefaultColorSelected() {
        return chooser.isDefaultColorSelected();
    }

    public GedColor getSelectedColor() {
        return chooser.getSelectedColor();
    }

    public void setDefaultButtonColor(GedColor gedColor) {
        chooser.setDefaultButtonColor(gedColor);
    }

    public void addChangeValueAction(Runnable runnable) {
        valueField.addKeyReleasedAction(runnable);
        valueField.addActionListener(e -> runnable.run());
    }

    public void addChangeXDistanceAction(Runnable runnable) {
        xDistanceField.addKeyReleasedAction(runnable);
        xDistanceField.addActionListener(e -> runnable.run());
    }

    public void addChangeYDistanceAction(Runnable runnable) {
        yDistanceField.addKeyReleasedAction(runnable);
        yDistanceField.addActionListener(e -> runnable.run());
    }

}
