package de.duehl.math.graph.ged.graph.io;

import static org.junit.jupiter.api.Assertions.*;

import java.util.List;

import org.junit.Test;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.math.graph.ged.graph.io.ged4.Variable;

public class VariableTest {

    @Test
    public void fillAndGetName() {
        Variable var = new Variable("Name", "Wert");
        String actual = var.getName();
        String expected = "Name";
        assertEquals(expected, actual);
    }

    @Test
    public void fillAndGetValue() {
        Variable var = new Variable("Name", "Wert");
        String actual = var.getValue();
        String expected = "Wert";
        assertEquals(expected, actual);
    }

    @Test
    public void fillAndToString() {
        Variable var = new Variable("Name", "Wert");
        String actual = var.toString();
        String expected = "Name=Wert";
        assertEquals(expected, actual);
    }

    @Test
    public void splitVariableByEqualsSign() {
        Variable var = Variable.splitVariableByEqualsSign("Name=Wert");

        String actual1 = var.getName();
        String expected1 = "Name";
        assertEquals(expected1, actual1);

        String actual2 = var.getValue();
        String expected2 = "Wert";
        assertEquals(expected2, actual2);
    }

    @Test
    public void splitVariableByEqualsSignWithSpace() {
        Variable var = Variable.splitVariableByEqualsSign("Name =    Wert");

        String actual1 = var.getName();
        String expected1 = "Name";
        assertEquals(expected1, actual1);

        String actual2 = var.getValue();
        String expected2 = "Wert";
        assertEquals(expected2, actual2);
    }

    @Test
    public void splitVariableByColon() {
        Variable var = Variable.splitVariableByColon("Name:Wert");

        String actual1 = var.getName();
        String expected1 = "Name";
        assertEquals(expected1, actual1);

        String actual2 = var.getValue();
        String expected2 = "Wert";
        assertEquals(expected2, actual2);
    }

    @Test
    public void splitVariableByColonWithSpace() {
        Variable var = Variable.splitVariableByColon("Name :    Wert");

        String actual1 = var.getName();
        String expected1 = "Name";
        assertEquals(expected1, actual1);

        String actual2 = var.getValue();
        String expected2 = "Wert";
        assertEquals(expected2, actual2);
    }

    @Test
    public void variableListToEquationList() {
        List<Variable> list = CollectionsHelper.buildListFrom(
                new Variable("name1", "wert1"),
                new Variable("name2", "wert2"),
                new Variable("name3", "wert3")
                );
        List<String> actual = Variable.variableListToEquationList(list);
        assertEquals(3, actual.size());
        assertEquals("name1=wert1", actual.get(0));
        assertEquals("name2=wert2", actual.get(1));
        assertEquals("name3=wert3", actual.get(2));
    }

    @Test
    public void variableListToSemicolonJoinedEquationList() {
        List<Variable> list = CollectionsHelper.buildListFrom(
                new Variable("name1", "wert1"),
                new Variable("name2", "wert2"),
                new Variable("name3", "wert3")
                );
        String actual = Variable.variableListToSemicolonJoinedEquationList(list);
        String expected = "name1=wert1;name2=wert2;name3=wert3";
        assertEquals(expected, actual);
    }

}
