package de.duehl.math.graph.ged.graph;

/**
 * Diese Klasse enthält die Metadaten zur Darstellung eines Graphen, die nichts mit der
 * mathematischen Datenvorhaltung der Ecken und Kanten zu tun haben.
 *
 * @version 1.01     2019-05-21
 * @author Christian Dühl
 */

public class MetaData {

    private GedColor defaultVertexColor;
    private GedColor defaultEdgeColor;
    private int defaultRadius;
    private GedColor backgroundColor;
    private boolean showGrid;
    private int gridDistance;
    private boolean snapInGrid;
    private GedColor gridColor;
    private int defaultEdgeWidth;
    private String backgroundPicture;

    /** Konstruktor */
    public MetaData() {
        clear();
    }

    public MetaData(MetaData that) {
        defaultVertexColor = that.defaultVertexColor;
        defaultEdgeColor   = that.defaultEdgeColor;
        defaultRadius      = that.defaultRadius;
        backgroundColor    = that.backgroundColor;
        showGrid           = that.showGrid;
        gridDistance       = that.gridDistance;
        snapInGrid         = that.snapInGrid;
        gridColor          = that.gridColor;
        defaultEdgeWidth   = that.defaultEdgeWidth;
        backgroundPicture  = that.backgroundPicture;
    }

    public void clear() {
        defaultVertexColor = GedColor.RED;
        defaultEdgeColor   = GedColor.BLACK;
        defaultRadius      = 4;
        backgroundColor    = GedColor.WHITE;
        showGrid           = false;
        gridDistance       = 20;
        snapInGrid         = false;
        gridColor          = GedColor.LIGHT_GRAY;
        defaultEdgeWidth   = 1;
        backgroundPicture  = "";
    }


    /* Getter */

    public GedColor getDefaultVertexColor() {
        return defaultVertexColor;
    }

    public GedColor getDefaultEdgeColor() {
        return defaultEdgeColor;
    }

    public int getDefaultRadius() {
        return defaultRadius;
    }

    public GedColor getBackgroundColor() {
        return backgroundColor;
    }

    public boolean isShowGrid() {
        return showGrid;
    }

    public int getGridDistance() {
        return gridDistance;
    }

    public boolean isSnapInGrid() {
        return snapInGrid;
    }

    public GedColor getGridColor() {
        return gridColor;
    }

    public int getDefaultEdgeWidth() {
        return defaultEdgeWidth;
    }

    public String getBackgroundPicture() {
        return backgroundPicture;
    }


    /* Setter */

    public void setDefaultVertexColor(GedColor defaultVertexColor) {
        this.defaultVertexColor = defaultVertexColor;
    }

    public void setDefaultEdgeColor(GedColor defaultEdgeColor) {
        this.defaultEdgeColor = defaultEdgeColor;
    }

    public void setDefaultRadius(int defaultRadius) {
        this.defaultRadius = defaultRadius;
    }

    public void setBackgroundColor(GedColor backgroundColor) {
        this.backgroundColor = backgroundColor;
    }

    public void setShowGrid(boolean showGrid) {
        this.showGrid = showGrid;
    }

    public void setGridDistance(int gridDistance) {
        this.gridDistance = gridDistance;
    }

    public void setSnapInGrid(boolean snapInGrid) {
        this.snapInGrid = snapInGrid;
    }

    public void setGridColor(GedColor gridColor) {
        this.gridColor = gridColor;
    }

    public void setDefaultEdgeWidth(int defaultEdgeWidth) {
        this.defaultEdgeWidth = defaultEdgeWidth;
    }

    public void setBackgroundPicture(String backgroundPicture) {
        this.backgroundPicture = backgroundPicture;
    }

    @Override
    public String toString() {
        return "MetaData:\n"
                + "    Default Eckenfarbe : " + defaultVertexColor + "\n"
                + "    Default Kantenfarbe: " + defaultEdgeColor + "\n"
                + "    Default Radius     : " + defaultRadius + "\n"
                + "    Hintergrundfarbe   : " + backgroundColor + "\n"
                + "    Hintergrundbild    : " + backgroundPicture + "\n"
                + "    Gitter anzeigen    : " + showGrid + "\n"
                + "    Gitterabstand      : " + gridDistance + "\n"
                + "    Einrasten am Gitter: " + snapInGrid + "\n"
                + "    Farbe des Gitters  : " + gridColor + "\n"
                + "    Default Kantendicke: " + defaultEdgeWidth + "\n";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((backgroundColor == null) ? 0 : backgroundColor.hashCode());
        result = prime * result + ((backgroundPicture == null) ? 0 : backgroundPicture.hashCode());
        result = prime * result + ((defaultEdgeColor == null) ? 0 : defaultEdgeColor.hashCode());
        result = prime * result + defaultEdgeWidth;
        result = prime * result + defaultRadius;
        result = prime * result
                + ((defaultVertexColor == null) ? 0 : defaultVertexColor.hashCode());
        result = prime * result + ((gridColor == null) ? 0 : gridColor.hashCode());
        result = prime * result + gridDistance;
        result = prime * result + (showGrid ? 1231 : 1237);
        result = prime * result + (snapInGrid ? 1231 : 1237);
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        MetaData other = (MetaData) obj;
        if (backgroundColor == null) {
            if (other.backgroundColor != null)
                return false;
        }
        else if (!backgroundColor.equals(other.backgroundColor))
            return false;
        if (backgroundPicture == null) {
            if (other.backgroundPicture != null)
                return false;
        }
        else if (!backgroundPicture.equals(other.backgroundPicture))
            return false;
        if (defaultEdgeColor == null) {
            if (other.defaultEdgeColor != null)
                return false;
        }
        else if (!defaultEdgeColor.equals(other.defaultEdgeColor))
            return false;
        if (defaultEdgeWidth != other.defaultEdgeWidth)
            return false;
        if (defaultRadius != other.defaultRadius)
            return false;
        if (defaultVertexColor == null) {
            if (other.defaultVertexColor != null)
                return false;
        }
        else if (!defaultVertexColor.equals(other.defaultVertexColor))
            return false;
        if (gridColor == null) {
            if (other.gridColor != null)
                return false;
        }
        else if (!gridColor.equals(other.gridColor))
            return false;
        if (gridDistance != other.gridDistance)
            return false;
        if (showGrid != other.showGrid)
            return false;
        if (snapInGrid != other.snapInGrid)
            return false;
        return true;
    }

}
