package de.duehl.math.graph.ged.graph.edge;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Diese Klasse stellt die Kanten eines Graphen in Form einer Menge von Adjazenzlisten dar.
 * In diesen Adjazenzlisten sind die Indices der benachbarten Ecken enthalten.
 *
 * @version 1.01     2019-05-20
 * @author Christian Dühl
 */

public class AdjacenceLists implements Iterable<AdjacenceList> {

    /**
     * Liste der Adjazenzlisten: Für jeden Knoten (für seinen Index) wird eine Liste der Nachfolger
     * entlang gerichteter Kanten abgespeichert.
     */
    private final List<AdjacenceList> adjacenceLists;

    public AdjacenceLists() {
        adjacenceLists = new ArrayList<>();
    }

    /**
     * Gibt die Anzahl der Kanten des Graphen zurück.
     *
     * @return Anzahl der (gerichteten) Kanten.
     */
    public int getNumberOfEdges() {
        int edgeCount = 0;
        for (AdjacenceList list : adjacenceLists) {
            edgeCount += list.size();
        }
        return edgeCount;
    }

    @Override
    public Iterator<AdjacenceList> iterator() {
        return adjacenceLists.iterator();
    }

    public void add(AdjacenceList list) {
        adjacenceLists.add(list);
    }

    public int size() {
        return adjacenceLists.size();
    }

    public AdjacenceList get(int index) {
        return adjacenceLists.get(index);
    }

    public void remove(int index) {
        adjacenceLists.remove(index);
    }

    public void clear() {
        adjacenceLists.clear();
    }

}
