package de.duehl.math.graph.ged.ui.dialogs;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Dialog.ModalExclusionType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Diese Klasse öffnet einen Dialog und zeigt einen Text in einem
 * Textfeld an.
 *
 * @version 1.02     2012-08-02
 * @author Christian Dühl
 */

public class ShowTextDialog extends JFrame implements ActionListener,
        KeyListener {

    private static final long serialVersionUID = 1L;

    private JButton quitButton;

    public ShowTextDialog(String title, String text) {
        super(title);
        addKeyListener(this);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setModalExclusionType(ModalExclusionType.NO_EXCLUDE);

        Container pane = getContentPane();
        pane.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets  = new Insets(0, 0, 0, 0);
        gbc.fill    = GridBagConstraints.NONE;
        gbc.anchor  = GridBagConstraints.WEST;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;

        gbc.gridx   = 0;
        gbc.gridy   = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill    = GridBagConstraints.BOTH;
        JTextArea area = new JTextArea();
        area.setEditable(false);
        area.setLineWrap(false);
        area.setWrapStyleWord(true);
        area.setText(text);
        area.setCaretPosition(0);
        area.setFont(new Font("Courier New", Font.PLAIN, 14)); // Serif
        JScrollPane scrollPane = new JScrollPane(area);
        scrollPane.setPreferredSize(new Dimension(500, 250));
        pane.add(scrollPane, gbc);

        gbc.gridx   = 0;
        gbc.gridy   = 1;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill    = GridBagConstraints.NONE;
        gbc.anchor  = GridBagConstraints.EAST;
        quitButton  = new JButton("Beenden");
        quitButton.addActionListener(this);
        pane.add(quitButton, gbc);

        setLocation(300, 200);
        pack();
        setVisible(true);
    }


    @Override
    public void actionPerformed(ActionEvent event) {
        if (quitButton == event.getSource()) {
            dispose();
            //this.setVisible(false);
        }
    }


    @Override
    public void keyPressed(KeyEvent e) {
        //int code = e.getKeyCode();
        //System.out.println("Taste gedrückt: " + code);
    }


    @Override
    public void keyReleased(KeyEvent e) {
        //int code = e.getKeyCode();
        //System.out.println("Taste losgelassen: " + code);
    }


    @Override
    public void keyTyped(KeyEvent e) {
        int code = e.getKeyCode();
        //System.out.println("Taste gedrückt und losgelassen: " + code);
        if (code == KeyEvent.VK_ESCAPE) {
            this.setVisible(false);
        }
    }

}


// -> http://forum.javacore.de/viewtopic.php?p=64724#64724