package de.duehl.math.sets;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.collections.OrderedSet;

/**
 * Diese Klasse erzeugt die Potenzmenge zu einer Menge.
 *
 * Siehe https://rosettacode.org/wiki/Power_set
 *
 * @version 1.01     2018-12-13
 * @author Christian Dühl
 */

public class PowerSet<Element> {

    private final List<OrderedSet<Element>> subsets;
    private OrderedSet<Element> set;
    private int n;

    public PowerSet(OrderedSet<Element> set) {
        this.set = set;
        n = set.size();
        subsets = new ArrayList<>();
    }

    public List<OrderedSet<Element>> create() {
        for (int k = 0; k <= n; ++k) {
            addSubsetsOfGivenSize(k);
        }

        return subsets;
    }

    private void addSubsetsOfGivenSize(int k) {
        if (k == 0) {
            OrderedSet<Element> emptySet = new OrderedSet<>();
            subsets.add(emptySet);
        }
        else if (k == n) {
            subsets.add(set);
        }
        // TODO Auto-generated method stub

    }

}
