package de.duehl.math.geometry;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import org.junit.Test;

import de.duehl.math.geometry.Circle;
import de.duehl.math.geometry.Point;

public class CircleTest {

    @Test
    public void create1() {
        assertNotNull(new Circle());
    }

    @Test
    public void create2() {
        assertNotNull(new Circle(new Point(1.0, 2.0), 17.0));
    }

    @Test
    public void getRadius() {
        Point center = new Point(1.0, 2.0);
        double radius = 17.0;
        Circle circle = new Circle(center, radius);
        double expected = radius;
        double actual = circle.getRadius();
        assertEquals(expected, actual, 0.000001);
    }

    @Test
    public void getCenter() {
        Point center = new Point(1.0, 2.0);
        double radius = 17.0;
        Circle circle = new Circle(center, radius);
        Point expected = center;
        Point actual = circle.getCenter();
        assertEquals(expected, actual);
    }

    @Test
    public void setRadius() {
        Point center = new Point(1.0, 2.0);
        double radius1 = 17.0;
        double radius2 = 19.0;
        Circle circle = new Circle(center, radius1);
        circle.setRadius(radius2);
        double expected = radius2;
        double actual = circle.getRadius();
        assertEquals(expected, actual, 0.000001);
    }

    @Test
    public void setCenter() {
        Point center1 = new Point(1.0, 2.0);
        Point center2 = new Point(3.0, 4.0);
        double radius = 17.0;
        Circle circle = new Circle(center1, radius);
        circle.setCenter(center2);
        Point expected = center2;
        Point actual = circle.getCenter();
        assertEquals(expected, actual);
    }

    @Test
    public void incloses1() {
        Point center = new Point(10.0, 10.0);
        double radius = 10.0;
        Circle circle = new Circle(center, radius);
        boolean expected = true;
        boolean actual = circle.incloses(new Point(3.0, 3.0));
        assertEquals(expected, actual);
    }

    @Test
    public void incloses2() {
        Point center = new Point(10.0, 10.0);
        double radius = 10.0;
        Circle circle = new Circle(center, radius);
        boolean expected = false;
        boolean actual = circle.incloses(new Point(2.0, 2.0));
        assertEquals(expected, actual);
    }

}
