package de.duehl.swing.ui.dialogs.html;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Dialog.ModalityType;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Image;
import java.awt.Point;

import javax.swing.JDialog;
import javax.swing.JFrame;

/**
 * Diese Klasse öffnet einen Dialog um einen Text in HTML in einem Textfeld anzuzeigen.
 *
 * @version 1.01     2016-07-29
 * @author Christian Dühl
 */

public final class ShowHTMLDialog extends ShowHtml {

    /** Das Hauptfenster des Programms. */
    private final JDialog dialog;

    /**
     * Konstruktor.
     *
     * @param title
     *            Titel des Dialogs.
     * @param programImage
     *            Image vom aufrufenden Programm.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    public ShowHTMLDialog(String title, Image programImage, Point parentLocation) {
        dialog = new JDialog();
        buildTheGui(title, programImage, parentLocation);
    }

    /**
     * Erzeugt die Oberfläche.
     *
     * @param title
     *            Titel des Dialogs.
     *
     * @param programImage
     *            Image vom aufrufenden Programm.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    private void buildTheGui(String title, Image programImage, Point parentLocation) {
        dialog.setTitle(title);
        dialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        dialog.setModalityType(ModalityType.APPLICATION_MODAL); // oder NO_EXCLUDE
        if (programImage != null) {
            dialog.setIconImage(programImage);
        }

        dialog.setLayout(new BorderLayout());

        dialog.add(getPanel(), BorderLayout.CENTER);

        dialog.setLocation(parentLocation.x + 50, parentLocation.y + 50);
        dialog.pack();

        setKeyBindings(dialog.getRootPane());
    }

    /** Schließt den Dialog. */
    @Override
    protected void closeDialog() {
        dialog.setVisible(false);
        dialog.dispose();
    }

    /**
     * Lässt den Dialog sichtbar oder unsichtbar werden.
     *
     * @param bool
     *            Gibt an, ob der Dialog zu sehen sein soll.
     */
    public void setVisible(boolean bool) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                dialog.setVisible(bool);
            }
        });
    }

    public void setPreferredSize(Dimension dimension) {
        dialog.setPreferredSize(dimension);
    }

}
