package de.duehl.swing.ui.dialogs.html;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Image;
import java.awt.Point;
import java.awt.Dialog.ModalExclusionType;

import javax.swing.JFrame;

/**
 * Diese Klasse öffnet einen JFrame um einen Text in HTML in einem Textfeld anzuzeigen.
 *
 * @version 1.01     2017-06-08
 * @author Christian Dühl
 */

public final class ShowHTMLFrame extends ShowHtml {

    /** Das Hauptfenster des Programms. */
    private JFrame frame;

    /**
     * Konstruktor.
     *
     * @param title
     *            Titel des Dialogs.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    public ShowHTMLFrame(String title) {
        this(title, new Point(100, 100));
    }

    /**
     * Konstruktor.
     *
     * @param title
     *            Titel des Dialogs.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    public ShowHTMLFrame(String title, Point parentLocation) {
        this(title, null, parentLocation);
    }

    /**
     * Konstruktor.
     *
     * @param title
     *            Titel des Dialogs.
     * @param programImage
     *            Image vom aufrufenden Programm.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    public ShowHTMLFrame(String title, Image programImage, Point parentLocation) {
        buildTheGui(title, programImage, parentLocation);
    }

    /**
     * Erzeugt die Oberfläche.
     *
     * @param title
     *            Titel des Dialogs.
     * @param programImage
     *            Image vom aufrufenden Programm.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    private void buildTheGui(String title, Image programImage, Point parentLocation) {
        frame = new JFrame(title);
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.setModalExclusionType(ModalExclusionType.NO_EXCLUDE);
        if (programImage != null) {
            frame.setIconImage(programImage);
        }

        frame.setLayout(new BorderLayout());

        frame.add(getPanel(), BorderLayout.CENTER);

        frame.setLocation(parentLocation.x + 50, parentLocation.y + 50);
        frame.pack();

        setKeyBindings(frame.getRootPane());
    }

    /** Schließt den Dialog. */
    @Override
    public void closeDialog() {
        frame.setVisible(false);
        frame.dispose();
    }

    /**
     * Lässt den Dialog sichtbar oder unsichtbar werden.
     *
     * @param bool
     *            Gibt an, ob der Dialog zu sehen sein soll.
     */
    public void setVisible(boolean bool) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                frame.setVisible(bool);
            }
        });
    }

    public void setPreferredSize(Dimension dimension) {
        frame.setPreferredSize(dimension);
    }

}
